<?php

declare(strict_types=1);

namespace mirzaev\ebala\controllers;

// Файлы проекта
use mirzaev\ebala\controllers\core,
	mirzaev\ebala\controllers\traits\errors,
	mirzaev\ebala\models\market as model,

// Библиотека для ArangoDB
use ArangoDBClient\Document as _document;

// System libraries
use datetime,
	exception;

/**
 * Контроллер магазина
 *
 * @package mirzaev\ebala\controllers
 * @author Arsen Mirzaev Tatyano-Muradovich <arsen@mirzaev.sexy>
 */
final class market extends core
{
	use errors;

	/**
	 * Главная страница
	 *
	 * @param array $parameters Параметры запроса
	 */
	public function index(array $parameters = []): ?string
	{
		// Авторизация
		if ($this->account->status() && ($this->account->type === 'administrator' || $this->account->type === 'operator')) {
			// Авторизован аккаунт оператора или администратора

			foreach (['confirmed', 'waiting', 'published', 'unpublished', 'problematic', 'hided', 'completed'] as $name) {
				// Перебор фильтров статусов

				// Инициализация значения (приоритет у cookie)
				$value = $_COOKIE["markets_filter_$name"] ?? $this->session->buffer[$_SERVER['INTERFACE']]['markets']['filters'][$name] ?? 0;

				// Инициализировано значение?
				if ($value === null || $value === 0) continue;

				// Генерация класса для HTML-элемента по его состоянию (0 - ОТСУТСТВУЕТ, 1 - И, 2 - ИЛИ)
				$this->view->{$name} = match ($value) {
					'0', 0 => 'earth',
					'1', 1 => 'sand',
					'2', 2 => 'river',
					default => 'earth'
				};
			}

			// Генерация представлениямя
			$main = $this->view->render(DIRECTORY_SEPARATOR . 'pages' . DIRECTORY_SEPARATOR . 'markets.html');
		} else $main = $this->authorization();

		// Возврат (успех)
		if ($_SERVER['REQUEST_METHOD'] === 'GET') return $this->view->render(DIRECTORY_SEPARATOR . 'index.html', ['main' => $main]);
		else if ($_SERVER['REQUEST_METHOD'] === 'POST') return $main;

		// Возврат (провал)
		return null;
	}

	/**
	 * Прочитать
	 *
	 * @param array $parameters Параметры запроса
	 */
	public function read(array $parameters = []): ?string
	{
		if ($this->account->status() && ($this->account->type === 'administrator' || $this->account->type === 'operator')) {
			// Авторизован аккаунт оператора или администратора

			// Реинициализация актуальной страницы
			if (isset($parameters['page'])) $this->session->write(['markets' => ['page' => $parameters['page']]]);
			else if (empty($this->session->buffer[$_SERVER['INTERFACE']]['markets']['page'])) $this->session->write(['markets' => ['page' => 1]]);

			// Инициализация буфера AQL-выражения для инъекции фильтра по интервалу
			$polysemantic = '';

			// Инициализация допустимых статусов
			$statuses = ['active', 'inactive', 'fined', 'decent', 'hided', 'fired'];

			// Инициализация буфера AQL-выражения для инъекции фильтра по статусам (И)
			$statuses_and = '';

			foreach ($statuses as $name) {
				// Перебор фильтров статусов (И)

				// Инициализация значения (приоритет у cookie) (отсутствие значения или значение 0 вызывают continue)
				if (empty($value = $_COOKIE["markets_filter_$name"] ?? $this->session->buffer[$_SERVER['INTERFACE']]['markets']['filters'][$name] ?? 0)) continue;

				// Генерация AQL-выражения для инъекции в строку запроса
				if ($value === '1') $statuses_and .= " && market.$name == true";
			}

			// Очистка от бинарных операторов сравнения с только одним операндом (крайние)
			$statuses_and = trim(trim(trim($statuses_and), '&&'));

			// Инициализация буфера AQL-выражения для инъекции фильтра по статусам (ИЛИ)
			$statuses_or = '';

			foreach ($statuses as $name) {
				// Перебор фильтров статусов (ИЛИ)

				// Инициализация значения (приоритет у cookie) (отсутствие значения или значение 0 вызывают continue)
				if (empty($value = $_COOKIE["markets_filter_$name"] ?? $this->session->buffer[$_SERVER['INTERFACE']]['markets']['filters'][$name] ?? 0)) continue;

				// Генерация AQL-выражения для инъекции в строку запроса
				if ($value === '2') $statuses_or .= " || market.$name == true";
			}

			// Очистка от бинарных операторов сравнения с только одним операндом (крайние)
			$statuses_or = trim(trim(trim($statuses_or), '||'));

			// Инициализация буфера с объёдинёнными буферами c AQL-выражениям "И" и "ИЛИ"
			$statuses_merged = (empty($statuses_and) ? '' : "($statuses_and)") . (empty($statuses_or) ? '' : (empty($statuses_and) ? '' : ' || ') . "($statuses_or)");

			// Инициализация общего буфера с AQL-выражениями
			$filters = '';

			// Объединение фильров в единую строку с AQL-выражениями для инъекции
			if (!empty($statuses_merged)) $filters .= empty($filters) ? $statuses_merged : " && ($statuses_merged)";

			// Инициализация данных для генерации HTML-документа с таблицей
			$this->view->rows = model::list(before: empty($filters) ? null : "FILTER ($filters)", page: (int) $this->session->buffer[$_SERVER['INTERFACE']]['markets']['page']);

			// Запись в cookie (только таким методом можно записать "hostonly: true")		
			setcookie(
				'markets_page',
				(string) $this->session->buffer[$_SERVER['INTERFACE']]['markets']['page'],
				[
					'expires' => strtotime('+1 hour'),
					'path' => '/',
					'secure' => true,
					'httponly' => false,
					'samesite' => 'strict'
				]
			);

			// Запись в глобальную переменную шаблонизатора обрабатываемой страницы
			$this->view->page = $parameters['page'];

			// Инициализация блока
			return $this->view->render(DIRECTORY_SEPARATOR . 'elements' . DIRECTORY_SEPARATOR . 'markets.html');
		}

		// Возврат (провал)
		return null;
	}

	/**
	 * Прочитать данные магазинов для <datalist>
	 *
	 * @param array $parameters Параметры запроса
	 */
	public function datalist(array $parameters = []): ?string
	{
		if ($this->account->status() && ($this->account->type === 'administrator' || $this->account->type === 'operator' || $this->account->type === 'market')) {
			// Авторизован аккаунт оператора или магазина

			// Инициализация данных магазинов
			$this->view->markets = model::read(filter: 'd.status == "active"', amount: 10000, return: '{ id: d.id, director: d.director }');

			// Универсализация
			if ($this->view->markets instanceof _document) $this->view->markets = [$this->view->markets];

			// Возврат (успех)
			return $this->view->render(DIRECTORY_SEPARATOR . 'lists' . DIRECTORY_SEPARATOR . 'markets.html');
		}

		// Возврат (провал)
		return null;
	}
}
