"use strict";

if (typeof window.tasks !== "function") {
  // Not initialized

  // Initialize of the class in global namespace
  window.tasks = class tasks {
    /**
     * Заблокировать функцию закрытия всплывающего окна?
     */
    static freeze = false;

    /**
     * Тело всплывающего окна (массив)
     */
    static body = {};

    /**
     * Инициализирован класс?
     */
    static initialized = false;

    /**
     * Создать заявку
     *
     * @param {HTMLElement} cashiers Количество кассиров <input>
     * @param {HTMLElement} displayers Количество выкладчиков <input>
     * @param {HTMLElement} loaders Количество грузчиков <input>
     * @param {HTMLElement} gastronomes Количество гастрономов <input>
     * @param {HTMLElement} start Начало работы (00:00) <input>
     * @param {HTMLElement} end Конец работы (00:00) <input>
     * @param {HTMLElement} date Дата работы (d.m.Y) <input>
     * @param {HTMLElement} button Кнопка создания заявки <button>
     *
     * @return {void}
     */
    static _create(
      cashiers,
      displayers,
      loaders,
      gastronomes,
      start,
      end,
      date,
      button,
    ) {
      // Блокировка полей ввода
      cashiers.setAttribute("readonly", true);
      displayers.setAttribute("readonly", true);
      loaders.setAttribute("readonly", true);
      gastronomes.setAttribute("readonly", true);
      start.setAttribute("readonly", true);
      end.setAttribute("readonly", true);
      date.setAttribute("readonly", true);

      // Блокировка кнопки создания
      button.setAttribute("disabled", true);

      // Запуск выполнения
      this.__create(
        cashiers,
        displayers,
        loaders,
        gastronomes,
        start,
        end,
        date,
        button,
      );
    }

    /**
     * Создать заявку
     *
     * @param {HTMLElement} cashiers Количество кассиров <input>
     * @param {HTMLElement} displayers Количество выкладчиков <input>
     * @param {HTMLElement} loaders Количество грузчиков <input>
     * @param {HTMLElement} gastronomes Количество гастрономов <input>
     * @param {HTMLElement} start Начало работы (00:00) <input>
     * @param {HTMLElement} end Конец работы (00:00) <input>
     * @param {HTMLElement} date Дата работы (d.m.Y) <input>
     * @param {HTMLElement} button Кнопка создания заявки <button>
     *
     * @return {void}
     */
    static __create = damper(
      async (
        cashiers,
        displayers,
        loaders,
        gastronomes,
        start,
        end,
        date,
        button,
      ) => {
        // Инициализация функции разблокировки
        function unblock() {
          // Разблокировка полей ввода
          cashiers.removeAttribute("readonly");
          displayers.removeAttribute("readonly");
          loaders.removeAttribute("readonly");
          gastronomes.removeAttribute("readonly");
          start.removeAttribute("readonly");
          end.removeAttribute("readonly");
          date.removeAttribute("readonly");

          // Разблокировка кнопки
          button.removeAttribute("disabled");
        }

        // Запуск отсрочки разблокировки на случай, если сервер не отвечает
        const timeout = setTimeout(() => {
          this.errors(["Сервер не отвечает"]);
          unblock();
        }, 5000);

        // Запрос к серверу
        return await fetch("/tasks/create", {
          method: "POST",
          headers: {
            "Content-Type": "application/x-www-form-urlencoded",
          },
          body:
            `cashiers=${cashiers.value}&displayers=${displayers.value}&loaders=${loaders.value}&gastronomes=${gastronomes.value}&start=${start.value}&end=${end.value}&date=${
              date.valueAsDate / 1000
            }`,
        })
          .then((response) => response.json())
          .then((data) => {
            // Удаление отсрочки разблокировки
            clearTimeout(timeout);

            if (this.errors(data.errors)) {
              // Сгенерированы ошибки

              // Разблокировка полей ввода и кнопок
              unblock();
            } else {
              // Не сгенерированы ошибки (подразумевается их отсутствие)

              // Деинициализация всплывающего окна
              if (this.body.wrap instanceof HTMLElement) {
                this.body.wrap.remove();
              }

              // Реинициализация строк
              this.reinit();
            }
          });
      },
      300,
    );

    /**
     * @param {HTMLElement} row Строка
     *
     * @return {void}
     */
    static create = damper(() => {
      // Инициализация оболочки всплывающего окна
      this.body.wrap = document.createElement("div");
      this.body.wrap.setAttribute("id", "popup");

      // Инициализация всплывающего окна
      const popup = document.createElement("section");
      popup.classList.add("list", "small");

      // Инициализация заголовка всплывающего окна
      const title = document.createElement("h3");
      title.innerText = "Создание заявки";

      // Инициализация строки
      const row_1 = document.createElement("div");
      row_1.classList.add("row");

      // Инициализация оболочки для строки с кассирами
      const cashier = document.createElement("label");
      cashier.setAttribute("id", "cashier");

      // Инициализация заголовка для поля ввода кассиров
      const cashier_title = document.createElement("b");
      cashier_title.classList.add("separated", "right");
      cashier_title.innerText = "Кассиры:";

      // Инициализация поля ввода кассиров
      const cashier_input = document.createElement("input");
      cashier_input.classList.add("small", "center");
      cashier_input.setAttribute("type", "number");
      cashier_input.setAttribute("min", "0");
      cashier_input.setAttribute("max", "50");
      cashier_input.setAttribute(
        "onkeydown",
        "if (event.keyCode === 13) this.parentElement.parentElement.nextElementSibling.nextElementSibling.nextElementSibling.nextElementSibling.children[0].children[0].click()",
      );
      cashier_input.setAttribute("title", "Количество кассиров");
      cashier_input.value = 0;

      // Инициализация строки
      const row_2 = document.createElement("div");
      row_2.classList.add("row", "merged");

      // Инициализация оболочки для строки с выкладчиками
      const displayer = document.createElement("label");
      displayer.setAttribute("id", "displayer");

      // Инициализация заголовка для поля ввода выкладчиков
      const displayer_title = document.createElement("b");
      displayer_title.classList.add("separated", "right");
      displayer_title.innerText = "Выкладчики:";

      // Инициализация поля ввода выкладчиков
      const displayer_input = document.createElement("input");
      displayer_input.classList.add("small", "center");
      displayer_input.setAttribute("type", "number");
      displayer_input.setAttribute("min", "0");
      displayer_input.setAttribute("max", "50");
      displayer_input.setAttribute(
        "onkeydown",
        "if (event.keyCode === 13) this.parentElement.parentElement.nextElementSibling.nextElementSibling.nextElementSibling.children[0].children[0].click()",
      );
      displayer_input.setAttribute("title", "Количество выкладчиков");
      displayer_input.value = 0;

      // Инициализация строки
      const row_3 = document.createElement("div");
      row_3.classList.add("row", "merged");

      // Инициализация оболочки для строки с грузчиками
      const loader = document.createElement("label");
      loader.setAttribute("id", "loader");

      // Инициализация заголовка для поля ввода грузчиков
      const loader_title = document.createElement("b");
      loader_title.classList.add("separated", "right");
      loader_title.innerText = "Грузчики:";

      // Инициализация поля ввода грузчиков
      const loader_input = document.createElement("input");
      loader_input.classList.add("small", "center");
      loader_input.setAttribute("type", "number");
      loader_input.setAttribute("min", "0");
      loader_input.setAttribute("max", "50");
      loader_input.setAttribute(
        "onkeydown",
        "if (event.keyCode === 13) this.parentElement.parentElement.nextElementSibling.nextElementSibling.children[0].children[0].click()",
      );
      loader_input.setAttribute("title", "Количество грузчиков");
      loader_input.value = 0;

      // Инициализация строки
      const row_4 = document.createElement("div");
      row_4.classList.add("row", "merged");

      // Инициализация оболочки для строки с гастрономами
      const gastronome = document.createElement("label");
      gastronome.setAttribute("id", "gastronome");

      // Инициализация заголовка для поля ввода гастрономов
      const gastronome_title = document.createElement("b");
      gastronome_title.classList.add("separated", "right");
      gastronome_title.innerText = "Гастрономы:";

      // Инициализация поля ввода гастрономов
      const gastronome_input = document.createElement("input");
      gastronome_input.classList.add("small", "center");
      gastronome_input.setAttribute("type", "number");
      gastronome_input.setAttribute("min", "0");
      gastronome_input.setAttribute("max", "50");
      gastronome_input.setAttribute(
        "onkeydown",
        "if (event.keyCode === 13) this.parentElement.parentElement.nextElementSibling.children[0].children[0].click()",
      );
      gastronome_input.setAttribute("title", "Количество гастрономов");
      gastronome_input.value = 0;

      // Инициализация строки
      const row_5 = document.createElement("div");
      row_5.classList.add("row", "divided", "buttons");

      // Инициализация оболочки для кнопок
      const buttons = document.createElement("label");

      // Инициализация поля ввода времени начала
      const start_input = document.createElement("input");
      start_input.setAttribute("type", "time");
      start_input.setAttribute(
        "onkeydown",
        "if (event.keyCode === 13) this.nextElementSibling.nextElementSibling.nextElementSiblin.click()",
      );
      start_input.setAttribute("title", "Время начала работы по заявке");
      start_input.value = "09:00";

      // Инициализация поля ввода времени окончания
      const end_input = document.createElement("input");
      end_input.setAttribute("type", "time");
      end_input.setAttribute(
        "onkeydown",
        "if (event.keyCode === 13) this.nextElementSibling.nextElementSibling.click()",
      );
      end_input.setAttribute("title", "Время конца работы по заявке");
      end_input.value = "18:00";

      // Инициализация поля ввода даты (дня)
      const day_input = document.createElement("input");
      day_input.setAttribute("type", "date");
      day_input.setAttribute(
        "onkeydown",
        "if (event.keyCode === 13) this.nextElementSibling.click()",
      );
      day_input.setAttribute("title", "Дата заявки");
      const tomorrow = new Date();
      tomorrow.setDate(tomorrow.getDate() + 1);
      day_input.valueAsDate = tomorrow;

      // Инициализация кнопки подтверждения даты
      const create = document.createElement("button");
      create.classList.add("grass");
      create.innerText = "Создать";
      create.setAttribute(
        "onclick",
        `tasks._create(this.parentElement.parentElement.previousElementSibling.previousElementSibling.previousElementSibling.previousElementSibling.children[0].children[1], this.parentElement.parentElement.previousElementSibling.previousElementSibling.previousElementSibling.children[0].children[1], this.parentElement.parentElement.previousElementSibling.previousElementSibling.children[0].children[1], this.parentElement.parentElement.previousElementSibling.children[0].children[1], this.previousElementSibling.previousElementSibling.previousElementSibling, this.previousElementSibling.previousElementSibling, this.previousElementSibling, this)`,
      );

      // Инициализация окна с ошибками
      this.body.errors = document.createElement("section");
      this.body.errors.classList.add(
        "errors",
        "window",
        "list",
        "small",
        "hidden",
      );

      // Инициализация элемента-тела (оболочки) окна с ошибками
      const errors = document.createElement("section");
      errors.classList.add("body");

      // Инициализация элемента-списка ошибок
      const dl = document.createElement("dl");

      // Инициализация устаревшего всплывающего окна
      const old = document.getElementById("popup");

      // Удаление устаревшего всплывающего окна
      if (old instanceof HTMLElement) old.remove();

      // Запись в документ
      popup.appendChild(title);

      cashier.appendChild(cashier_title);
      cashier.appendChild(cashier_input);
      row_1.appendChild(cashier);
      popup.appendChild(row_1);

      displayer.appendChild(displayer_title);
      displayer.appendChild(displayer_input);
      row_2.appendChild(displayer);
      popup.appendChild(row_2);

      loader.appendChild(loader_title);
      loader.appendChild(loader_input);
      row_3.appendChild(loader);
      popup.appendChild(row_3);

      gastronome.appendChild(gastronome_title);
      gastronome.appendChild(gastronome_input);
      row_4.appendChild(gastronome);
      popup.appendChild(row_4);

      buttons.appendChild(start_input);
      buttons.appendChild(end_input);
      buttons.appendChild(day_input);
      buttons.appendChild(create);
      row_5.appendChild(buttons);
      popup.appendChild(row_5);

      this.body.wrap.appendChild(popup);
      document.body.appendChild(this.body.wrap);

      errors.appendChild(dl);
      this.body.errors.appendChild(errors);
      this.body.wrap.appendChild(this.body.errors);

      // Фокусировка
      cashier_input.focus();

      // Инициализация переменных для окна с ошибками (12 - это значение gap из div#popup)
      function top(errors) {
        errors.style.setProperty(
          "transition",
          "0s",
        );
        errors.style.setProperty(
          "--top",
          popup.offsetTop + popup.offsetHeight + 12 + "px",
        );
        setTimeout(
          () => errors.style.removeProperty("transition"),
          100,
        );
      }
      top(this.body.errors);
      const resize = new ResizeObserver(() => top(this.body.errors));
      resize.observe(this.body.wrap);

      // Инициализация функции блокировки кнопки
      const block = () => {
        if (
          cashier_input.value > 0 ||
          displayer_input.value > 0 ||
          loader_input.value > 0 ||
          gastronome_input.value > 0
        ) create.removeAttribute("disabled");
        else create.setAttribute("disabled", "true");
      };

      // Добавление функции блокировки кнопки по событиям
      cashier_input.addEventListener("keyup", block);
      displayer_input.addEventListener("keyup", block);
      loader_input.addEventListener("keyup", block);
      gastronome_input.addEventListener("keyup", block);
      cashier_input.addEventListener("change", block);
      displayer_input.addEventListener("change", block);
      loader_input.addEventListener("change", block);
      gastronome_input.addEventListener("change", block);

      // Первичная активация функции блокировки кнопки
      block();

      // Инициализация функции закрытия всплывающего окна
      const click = () => {
        // Удаление всплывающего окна
        this.body.wrap.remove();
      };

      // Инициализация функции добавления функции закрытия всплывающего окна
      const enable = () => this.body.wrap.addEventListener("click", click);

      // Инициализация функции удаления функции закрытия всплывающего окна
      const disable = () => this.body.wrap.removeEventListener("click", click);

      // Первичная активация функции удаления всплывающего окна
      enable();

      // Добавление функций удаления всплывающего окна по событиям
      popup.addEventListener("mouseenter", disable);
      popup.addEventListener("mouseleave", enable);
    }, 300);

    /**
     * Переключить фильтр
     *
     * @param {string} name Название
     * @param {string|number|null} value Значение
     * @param {HTMLElement|null} button Кнопка
     *
     * @return {void}
     */
    static filter = damper(async (name, value, button) => {
      if (typeof name === "string") {
        // Получено название

        // Инициализация сериализованного пути к директории
        const path = `tasks_filter_${name}`;

        if (typeof value === "string" || typeof value === "number") {
          // Получено значение

          // Запись нового значения
          buffer.write(path, isNaN(value) ? "" : value);
        } else {
          // Не получено значение

          // Чтение текущего значения
          value = +(await buffer.read(path));

          // Инициализация значения по умолчанию
          if (isNaN(value)) value = 0;

          // Запись нового значения (инвертирование)
          buffer.write(path, ++value < 3 ? value : 0);

          if (button instanceof HTMLElement) {
            // Получена кнопка

            // Деинициализация классов статуса фильтра
            button.classList.remove("earth", "sand", "river");

            // Инициализация классов статуса фильтра
            if (value === 1) button.classList.add("sand");
            else if (value === 2) button.classList.add("river");
            else button.classList.add("earth");
          }
        }
      }
    }, 300);

    /**
     * Инициализация страниц
     *
     * @param {number} amount Количество страниц для генерации
     * @param {number} iteration Номер итерации (системное)
     *
     * @return {void}
     */
    static async init(amount = 3, iteration = 0) {
      if (
        typeof amount === "number" && typeof iteration === "number" &&
        this.initialized === false
      ) {
        // Получены количество страниц и номер итерации

        // Инициализация страницы (написал tasks потому, что оптимизатор выдаёт ошибку, что this.read не найдено, хотя всё работает)
        await tasks.read(++iteration);

        // Проверка условий и отложенный запуск следующей итерации
        if (iteration < amount) setTimeout(this.init, 1500, amount, iteration);
        else this.initialized = true;
      }
    }

    /**
     * Реинициализация страниц
     *
     * @param {function} postprocessing Функция которая будет исполнена после реинициализации
     * @param {number} amount Количество страниц для генерации при инициализации
     *
     * @return {void}
     */
    static reinit = damper((postprocessing, amount = 3) => {
      if (typeof amount === "number") {
        // Получены количество страниц

        // Деинициализация cookie с номером страницы
        Cookies.remove("tasks_page", { path: "/" });

        // Инициализация оболочки
        const tasks = document.getElementById("tasks");

        if (tasks instanceof HTMLElement) {
          // Найдена оболочка

          // Инициализация буфера элементов
          const buffer = document.createElement("div");

          // Перенос элементов в буфер
          buffer.replaceChildren(...tasks.children);

          // Удаление комментариев с индексами страниц
          tasks.innerHTML = tasks.innerHTML.replace(
            /<!--(?!>)[\S\s]*?-->/g,
            "",
          );

          // Перенос элементов из буфера
          tasks.replaceChildren(...buffer.children);

          // Деинициализация страниц
          for (
            const element of tasks.querySelectorAll(
              ':scope > :not(form, div[id="title"])',
            )
          ) element.remove();

          // Сброс статуса инициализированности
          this.initialized = false;

          // Инициализация страниц
          this.init(amount);

          // Постобработка
          if (typeof postprocessing === "function") postprocessing();
        }
      }
    }, 2000);

    /**
     * Прочитать
     *
     * Читает список задач и модифицирует документ (записывает в общий список)
     *
     * @param {number} page Страница
     *
     * @return {object} {(bool) exist, (array) errors}
     */
    static read = damper(async (page) => {
      if (typeof core === "function" && typeof page !== "number") {
        // Не получена страница

        // Инициализация страницы (если не получена, то брать из cookie и прибавлять 1)
        page = Cookies.get(`tasks_page`) ?? 0;

        // Запись идентификатора следующей страницы
        ++page;
      }

      // Запрос к серверу
      return await fetch("/tasks/read", {
        method: "POST",
        headers: {
          "Content-Type": "application/x-www-form-urlencoded",
        },
        body: `page=${page}`,
      })
        .then((response) => response.text())
        .then((data) => {
          // Инициализация буфера для записи в документ
          const element = document.createElement("div");

          // Запись пустого элемента из буфера в документ
          document.getElementById("tasks").appendChild(element);

          // Запись в документ HTML-данных через буфер
          element.outerHTML = data;

          // Возврат (успех)
          return data;
        });
    }, 50);

    /**
     * Сгенерировать всплывающее окно c данными задачи
     *
     * @param {HTMLElement} row Строка
     *
     * @return {void}
     */
    static popup = damper(async (row) => {
      if (row instanceof HTMLElement) {
        // Получена строка

        // Инициализация идентификатора сотрудника
        const task = row.getAttribute("id");

        // Запрос к серверу
        return await fetch(`/task/${task}/read`, {
          method: "POST",
          headers: {
            "Content-Type": "application/x-www-form-urlencoded",
          },
        })
          .then((response) => response.text())
          .then((data) => {
            if (data.length > 0) {
              // Получены данные

              // Инициализация буфера выбранных строк
              const buffer = [
                ...document.querySelectorAll('div[data-selected="true"]'),
              ];

              // Удаление статуса активной строки у остальных строк
              for (const element of buffer) {
                element.removeAttribute("data-selected");
              }

              // Инициализация статуса активной строки (обрабатываемой в данный момент)
              row.setAttribute("data-selected", "true");

              // Инициализация оболочки всплывающего окна
              this.body.wrap = document.createElement("div");
              this.body.wrap.setAttribute("id", "popup");

              // Инициализация всплывающего окна
              const popup = document.createElement("section");
              popup.classList.add("list", "small");

              // Инициализация буфера для записи во всплывающее окно
              const list = document.createElement("div");

              // Инициализация оболочки для кнопок
              const _row = document.createElement("div");
              _row.classList.add("row", "divided");

              // Инициализация оболочки для строки с полем ввода даты
              const label = document.createElement("label");

              // Инициализация поля ввода даты
              const date = document.createElement("input");
              date.setAttribute("type", "date");
              date.setAttribute(
                "onkeydown",
                "if (event.keyCode === 13) this.nextElementSibling.nextElementSibling.nextElementSibling.click()",
              );
              date.setAttribute("title", "Дата заявки");
              this.value(row, "date").then((text) =>
                date.valueAsDate = new Date(text * 1000)
              );

              // Инициализация поля ввода времени начала
              const start = document.createElement("input");
              start.setAttribute("type", "time");
              start.setAttribute(
                "onkeydown",
                "if (event.keyCode === 13) this.nextElementSibling.nextElementSibling.click()",
              );
              start.setAttribute("title", "Время начала работы");
              this.value(row, "start").then((text) =>
                start.value = typeof text === "string"
                  ? (text.length === 7
                    ? 0 + text.replace(/(?<=:00):00/, "")
                    : text.replace(/(?<=:00):00/, ""))
                  : "00:00"
              );

              // Инициализация поля ввода времени окончания
              const end = document.createElement("input");
              end.setAttribute("type", "time");
              end.setAttribute(
                "onkeydown",
                "if (event.keyCode === 13) this.nextElementSibling.click()",
              );
              end.setAttribute("title", "Время конца работы");
              this.value(row, "end").then((text) =>
                end.value = typeof text === "string"
                  ? (text.length === 7
                    ? 0 + text.replace(/(?<=:00):00/, "")
                    : text.replace(/(?<=:00):00/, ""))
                  : "00:00"
              );

              // Инициализация кнопки подтверждения даты
              const replace = document.createElement("button");
              replace.classList.add("sea");
              replace.innerText = "Записать";
              replace.setAttribute(
                "onclick",
                `tasks.date(document.getElementById('${task}'), this.previousElementSibling.previousElementSibling.previousElementSibling, this.previousElementSibling.previousElementSibling, this.previousElementSibling, this)`,
              );

              // Инициализация списка выбора типа работы
              const work = document.createElement("select");
              work.classList.add("row", "connected");
              this.works(row).then((html) => work.innerHTML = html);
              work.setAttribute("title", "Тип работы");
              let changed = false;
              work.addEventListener("click", () => {
                if (changed) {
                  // Заблокировано выполнение onClick (подразумевается, что отработал onChange)

                  // Разблокировка следующего события onClick
                  changed = false;
                } else {
                  // Не заблокировано выполнение onClick (подразумевается, что нажали на элмент <select>)

                  // Блокировка выполнения закрытия всплывающего окна
                  this.freeze = true;
                }
              });
              work.addEventListener("change", () => {
                // Блокировка выполнения закрытия всплывающего окна (УЛОВКА ДЛЯ ОБХОДА СНЯТИЯ БЛОКИРОКИ ЧЕРЕЗ onmouseleave)
                this.freeze = true;

                // Блокировка события onClick (идёт после onChange)
                changed = true;

                // Разблокировка выполнения закрытия всплывающего окна с задержкой, чтобы оно сразу не закрылось
                setTimeout(() => this.freeze = false, 100);

                // Отправка значения на сервер
                this.work(row, work);
              });
              work.addEventListener("mouseout", () => this.freeze = false);

              // Инициализация поля ввода описания
              const description = document.createElement("textarea");
              this.value(row, "description").then((text) =>
                description.value = text
              );
              description.setAttribute("autofocus", "true");
              description.setAttribute("maxlength", "300");
              description.setAttribute(
                "onkeyup",
                `tasks.description(document.getElementById('${task}'), this)`,
              );
              description.setAttribute("title", "Дополнительная информация");
              description.setAttribute(
                "placeholder",
                "Дополнительная информация о требуемой работе",
              );

              // Инициализация оболочки для кнопок
              const buttons = document.createElement("div");
              buttons.classList.add("row", "divided", "buttons");

              // Инициализация кнопки подтверждения
              const confirm = document.createElement("button");
              confirm.classList.add("wide");
              if (row.classList.contains("confirmed")) {
                // Подтверждена заявка

                confirm.classList.add("clay");
                confirm.innerText = "Отклонить";
              } else {
                // Не подтверждена заявка

                confirm.classList.add("grass");
                confirm.innerText = "Подтвердить";
              }
              confirm.setAttribute(
                "onclick",
                `tasks.confirm(this, document.getElementById('${task}'))`,
              );

              // Инициализация кнопки скрытия
              const hide = document.createElement("button");
              if (row.classList.contains("hided")) {
                // Скрыта заявка

                hide.classList.add("grass");
                hide.innerText = "Показать";
              } else {
                // Не подтверждена заявка

                hide.classList.add("sea");
                hide.innerText = "Скрыть";
              }
              hide.setAttribute(
                "onclick",
                `tasks.hide(this, document.getElementById('${task}'))`,
              );

              // Инициализация кнопки удаления
              const remove = document.createElement("button");
              remove.classList.add("clay");
              remove.innerText = "Удалить";
              remove.setAttribute(
                "onclick",
                `tasks.remove(this, document.getElementById('${task}'))`,
              );

              // Инициализация окна с ошибками
              this.body.errors = document.createElement("section");
              this.body.errors.classList.add(
                "errors",
                "window",
                "list",
                "small",
                "hidden",
              );

              // Инициализация элемента-тела (оболочки) окна с ошибками
              const errors = document.createElement("section");
              errors.classList.add("body");

              // Инициализация элемента-списка ошибок
              const dl = document.createElement("dl");

              // Инициализация устаревшего всплывающего окна
              const old = document.getElementById("popup");

              // Удаление устаревшего всплывающего окна
              if (old instanceof HTMLElement) old.remove();

              // Удаление блокировки выполнения закрытия всплывающего окна
              this.freeze = false;

              // Запись в документ
              popup.appendChild(list);
              label.appendChild(date);
              label.appendChild(start);
              label.appendChild(end);
              label.appendChild(replace);
              _row.appendChild(label);
              popup.appendChild(_row);
              popup.appendChild(work);
              popup.appendChild(description);
              buttons.appendChild(confirm);
              buttons.appendChild(hide);
              buttons.appendChild(remove);
              popup.appendChild(buttons);
              this.body.wrap.appendChild(popup);
              errors.appendChild(dl);
              this.body.errors.appendChild(errors);
              this.body.wrap.appendChild(this.body.errors);
              document.body.appendChild(this.body.wrap);

              // Фокусировка
              description.focus();

              // Запись в документ HTML-данных через буфер
              list.outerHTML = data;

              // Инициализация переменных для окна с ошибками (12 - это значение gap из div#popup)
              function top(errors) {
                errors.style.setProperty(
                  "transition",
                  "0s",
                );
                errors.style.setProperty(
                  "--top",
                  popup.offsetTop + popup.offsetHeight + 12 + "px",
                );
                setTimeout(
                  () => errors.style.removeProperty("transition"),
                  100,
                );
              }
              top(this.body.errors);
              const resize = new ResizeObserver(() => top(this.body.errors));
              resize.observe(this.body.wrap);

              // Инициализация функции закрытия всплывающего окна
              const click = () => {
                // Блокировка выполнения
                if (this.freeze) return;

                // Удаление всплывающего окна
                this.body.wrap.remove();

                // Инициализация буфера выбранных строк
                const buffer = [
                  ...document.querySelectorAll('div[data-selected="true"]'),
                ];

                // Удаление статуса активной строки у остальных строк
                for (const element of buffer) {
                  element.removeAttribute("data-selected");
                }
              };

              // Инициализация функции добавления функции закрытия всплывающего окна
              const enable = () =>
                this.body.wrap.addEventListener("click", click);

              // Инициализация функции удаления функции закрытия всплывающего окна
              const disable = () =>
                this.body.wrap.removeEventListener("click", click);

              // Первичная активация функции удаления всплывающего окна
              enable();

              // Добавление функций удаления всплывающего окна по событиям
              popup.addEventListener("mouseenter", disable);
              popup.addEventListener("mouseleave", enable);
            }
          });
      }
    }, 300);

    /**
     * Подтвердить
     *
     * @param {HTMLElement} button Кнопка
     * @param {HTMLElement} row Строка
     *
     * @return {void}
     */
    static confirm = damper(async (button, row) => {
      if (row instanceof HTMLElement && button instanceof HTMLElement) {
        // Получена кнопка и строка

        // Инициализация идентификатора строки
        const id = row.getAttribute("id");

        if (typeof id === "string") {
          // Инициализирован идентификатор

          // Запрос к серверу
          return await fetch(`/task/${id}/confirm`, {
            method: "POST",
            headers: {
              "Content-Type": "application/x-www-form-urlencoded",
            },
          })
            .then((response) => response.json())
            .then((data) => {
              // Инициализация буффера списка классов
              const buffer = [...row.classList];

              // Реинициализация строки
              row.outerHTML = data.row;

              // Реинициализация строки (выражение странное, но правильное)
              row = document.getElementById(row.getAttribute("id"));

              // Копирование классов из буфера классов удалённой строки
              row.classList.add(...buffer);

              if (data.confirmed) {
                // Подтверждена заявка

                // Реинициализация текста
                button.innerText = "Отклонить";

                // Реинициализация стиля
                button.classList.remove("grass");
                button.classList.add("clay");

                // УЛОВКА: браузер ещё не успевает перерисовать документ во время исполнения this.task(row)
                row.classList.add("confirmed");
              } else {
                // Не подтверждена заявка

                // Реинициализация текста
                button.innerText = "Подтвердить";

                // Реинициализация стиля
                button.classList.remove("clay");
                button.classList.add("grass");

                // УЛОВКА: браузер ещё не успевает перерисовать документ во время исполнения this.task(row)
                row.classList.remove("confirmed");
              }

              // Реинициализация всплывающего окна
              if (this.body.wrap instanceof HTMLElement) {
                tasks.popup(row);
              }
            });
        }
      }
    }, 300);

    /**
     * Скрыть
     *
     * @param {HTMLElement} button Кнопка
     * @param {HTMLElement} row Строка
     *
     * @return {void}
     */
    static hide = damper(async (button, row) => {
      if (row instanceof HTMLElement && button instanceof HTMLElement) {
        // Получена кнопка и строка

        // Инициализация идентификатора строки
        const id = row.getAttribute("id");

        if (typeof id === "string") {
          // Инициализирован идентификатор

          // Запрос к серверу
          return await fetch(`/task/${id}/hide`, {
            method: "POST",
            headers: {
              "Content-Type": "application/x-www-form-urlencoded",
            },
          })
            .then((response) => response.json())
            .then((data) => {
              // Инициализация буффера списка классов
              const buffer = [...row.classList];

              // Реинициализация строки
              row.outerHTML = data.row;

              // Реинициализация строки (выражение странное, но правильное)
              row = document.getElementById(row.getAttribute("id"));

              // Копирование классов из буфера классов удалённой строки
              row.classList.add(...buffer);

              if (data.hided) {
                // Скрыта заявка

                // Реинициализация текста
                button.innerText = "Показать";

                // Реинициализация стиля
                button.classList.remove("sea");
                button.classList.add("grass");

                // УЛОВКА: браузер ещё не успевает перерисовать документ во время исполнения this.task(row)
                row.classList.add("hided");
              } else {
                // Не скрыта заявка

                // Реинициализация текста
                button.innerText = "Скрыть";

                // Реинициализация стиля
                button.classList.remove("grass");
                button.classList.add("sea");

                // УЛОВКА: браузер ещё не успевает перерисовать документ во время исполнения this.task(row)
                row.classList.remove("hided");
              }

              // Реинициализация всплывающего окна
              if (this.body.wrap instanceof HTMLElement) {
                tasks.popup(row);
              }
            });
        }
      }
    }, 300);

    /**
     * Удалить
     *
     * @param {HTMLElement} button Кнопка
     * @param {HTMLElement} row Строка
     *
     * @return {void}
     */
    static remove = damper(async (button, row) => {
      if (row instanceof HTMLElement && button instanceof HTMLElement) {
        // Получена кнопка и строка

        // Инициализация идентификатора строки
        const id = row.getAttribute("id");

        if (typeof id === "string") {
          // Инициализирован идентификатор

          // Запрос к серверу
          return await fetch(`/task/${id}/remove`, {
            method: "POST",
            headers: {
              "Content-Type": "application/x-www-form-urlencoded",
            },
          })
            .then((response) => response.json())
            .then((data) => {
              if (data.deleted) {
                // Удалена заявка

                // Удаление строки
                row.remove();

                // Деинициализация всплывающего окна
                if (this.body.wrap instanceof HTMLElement) {
                  this.body.wrap.remove();
                }
              }
            });
        }
      }
    }, 300);

    /**
     * Прочитать значение из базы данных
     *
     * @param {HTMLElement} row Строка
     * @param {string} name Название
     *
     * @return {string} Значение
     */
    static async value(row, name) {
      if (row instanceof HTMLElement && typeof name === "string") {
        // Получена строка и название

        // Инициализация идентификатора строки
        const id = row.getAttribute("id");

        if (typeof id === "string") {
          // Инициализирован идентификатор

          // Запрос к серверу
          return await fetch(`/task/${id}/value`, {
            method: "POST",
            headers: {
              "Content-Type": "application/x-www-form-urlencoded",
            },
            body: `name=${name}`,
          })
            .then((response) => response.text())
            .then((data) => {
              if (data.length > 0) {
                // Получены данные

                return data;
              }

              return null;
            });
        }
      }
    }

    /**
     * Сгенерировать список работ и выбрать в нём ту, что записана в базе данных у заявки
     *
     * @param {HTMLElement} row Строка
     *
     * @return {array|null} Массив HTML-элементов <option>
     */
    static async works(row) {
      if (row instanceof HTMLElement) {
        // Получена строка

        // Инициализация идентификатора строки
        const id = row.getAttribute("id");

        if (typeof id === "string") {
          // Инициализирован идентификатор

          // Запрос к серверу
          return await fetch(`/task/${id}/works`, {
            method: "POST",
            headers: {
              "Content-Type": "application/x-www-form-urlencoded",
            },
          })
            .then((response) => response.text())
            .then((data) => {
              if (data.length > 0) {
                // Получены данные

                return data;
              }

              return null;
            });
        }
      }
    }

    /**
     * Записать тип работы
     *
     * @param {HTMLElement} row Строка
     * @param {HTMLElement} select HTML-элемент <select>
     *
     * @return {void}
     */
    static work = damper(async (row, select) => {
      if (row instanceof HTMLElement && select instanceof HTMLElement) {
        // Получена строка и select-элемент

        // Инициализация идентификатора строки
        const id = row.getAttribute("id");

        if (typeof id === "string") {
          // Инициализирован идентификатор

          // Запрос к серверу
          return await fetch(`/task/${id}/work`, {
            method: "POST",
            headers: {
              "Content-Type": "application/x-www-form-urlencoded",
            },
            body: `work=${select.value}`,
          })
            .then((response) => response.json())
            .then((data) => {
              if (data.writed) {
                // Записано новое значение в базу данных

                // Инициализация буффера списка классов
                const buffer = [...row.classList];

                // Реинициализация строки в документе
                row.outerHTML = data.row;

                // Реинициализация строки (выражение странное, но правильное)
                row = document.getElementById(row.getAttribute("id"));

                // Копирование классов из буфера классов удалённой строки
                row.classList.add(...buffer);
              }

              // Реинициализация всплывающего окна
              if (this.body.wrap instanceof HTMLElement) {
                tasks.popup(row);
              }
            });
        }
      }
    }, 300);

    /**
     * Записать дату
     *
     * @param {HTMLElement} row Строка
     * @param {HTMLElement} date Дата
     * @param {HTMLElement} start Время начала
     * @param {HTMLElement} end Время окончания
     * @param {HTMLElement} button Кнопка отправки
     *
     * @return {void}
     */
    static date(row, date, start, end, button) {
      // Блокировка полей ввода
      date.setAttribute("readonly", true);
      start.setAttribute("readonly", true);
      end.setAttribute("readonly", true);
      button.setAttribute("disabled", true);

      // Запуск выполнения
      this._date(row, date, start, end, button);
    }

    /**
     * Записать дату
     *
     * @param {HTMLElement} row Строка
     * @param {HTMLElement} date Дата
     * @param {HTMLElement} start Время начала
     * @param {HTMLElement} end Время окончания
     * @param {HTMLElement} button Кнопка отправки
     *
     * @return {void}
     */
    static _date = damper(async (row, date, start, end, button) => {
      if (
        row instanceof HTMLElement && date instanceof HTMLElement &&
        start instanceof HTMLElement && end instanceof HTMLElement &&
        button instanceof HTMLElement
      ) {
        // Получены строка, поле даты, поле начала заявки, поле окончания заяки и кнопка

        // Инициализация идентификатора строки
        const id = row.getAttribute("id");

        if (typeof id === "string") {
          // Инициализирован идентификатор

          // Запрос к серверу
          return await fetch(`/task/${id}/date`, {
            method: "POST",
            headers: {
              "Content-Type": "application/x-www-form-urlencoded",
            },
            body: `date=${
              date.valueAsDate / 1000
            }&start=${start.value}&end=${end.value}`,
          })
            .then((response) => response.json())
            .then((data) => {
              if (data.writed) {
                // Записано новое значение в базу данных

                // Инициализация буффера списка классов
                const buffer = [...row.classList];

                // Реинициализация строки в документе
                row.outerHTML = data.row;

                // Реинициализация строки (выражение странное, но правильное)
                row = document.getElementById(row.getAttribute("id"));

                // Копирование классов из буфера классов удалённой строки
                row.classList.add(...buffer);

                // Реинициализация всплывающего окна
                if (this.body.wrap instanceof HTMLElement) {
                  tasks.popup(row);
                }
              } else {
                // Не записано новое значение в базу данных

                // Разблокировка ввода
                date.removeAttribute("readonly");
                start.removeAttribute("readonly");
                end.removeAttribute("readonly");
                button.removeAttribute("disabled");
              }
            });
        }
      }
    }, 300);

    /**
     * Записать описание
     *
     * @param {HTMLElement} row Строка
     * @param {HTMLElement} textarea HTML-элемент <textarea>
     *
     * @return {void}
     */
    static description(row, textarea) {
      // Сброс анимации ошибки
      textarea.classList.remove("error");

      // Запуск выполнения
      this._description(row, textarea);
    }

    /**
     * Записать описание (системное)
     *
     * @param {HTMLElement} row Строка
     * @param {HTMLElement} textarea HTML-элемент <textarea>
     *
     * @return {void}
     */
    static _description = damper(async (row, textarea) => {
      if (row instanceof HTMLElement && textarea instanceof HTMLElement) {
        // Получена строка и select-элемент

        // Инициализация идентификатора строки
        const id = row.getAttribute("id");

        if (typeof id === "string") {
          // Инициализирован идентификатор

          // Запрос к серверу
          return await fetch(`/task/${id}/description`, {
            method: "POST",
            headers: {
              "Content-Type": "application/x-www-form-urlencoded",
            },
            body: `description=${textarea.value}`,
          })
            .then((response) => response.json())
            .then((data) => {
              if (data.writed) {
                // Записано новое значение в базу данных

                // Удаление анимации ошибки
                textarea.classList.remove("error");
              } else {
                // Не записано новое значение в базу данных

                // Запись анимации ошибки
                textarea.classList.add("error");
              }
            });
        }
      }
    }, 300);

    /**
     * Сгенерировать HTML-элемент со списком ошибок
     *
     * @param {object} registry Реестр ошибок
     * @param {bool} reinitialization Реинициализировать?
     *
     * @return {bool} Сгенерированы ошибки?
     */
    static errors(registry, reinitialization = true) {
      // Инициализация ссылки на HTML-элемент с ошибками
      const wrap = this.body.errors;

      // Перерасчёт высоты элемента
      function height() {
        wrap.classList.remove("hidden");
        wrap.classList.remove("animation");
        // Реинициализация переменной с данными о высоте HTML-элемента (16 - это padding-top + padding-bottom у div#popup > section.errors)
        wrap.style.setProperty(
          "--height",
          wrap.offsetHeight - 16 + "px",
        );
        wrap.classList.add("animation");
        wrap.classList.add("hidden");
      }

      // Инициализация элемента-списка ошибок
      const list = wrap.getElementsByTagName("dl")[0];

      // Удаление ошибок из прошлой генерации
      if (reinitialization) list.innerHTML = null;

      for (const error in registry) {
        // Генерация HTML-элементов с текстами ошибок

        // Инициализация HTML-элемента текста ошибки
        const samp = document.createElement("samp");

        if (typeof registry[error] === "object") {
          // Категория ошибок

          // Проверка наличия ошибок
          if (registry[error].length === 0) continue;

          // Инициализация HTML-элемента-оболочки
          const wrap = document.createElement("dt");

          // Запись текста категории
          samp.innerText = error;

          // Запись HTML-элементов в список
          wrap.appendChild(samp);
          list.appendChild(wrap);

          // Реинициализация высоты
          height();

          // Обработка вложенных ошибок (вход в рекурсию)
          this.errors(registry[error], false);
        } else {
          // Текст ошибки (подразумевается)

          // Инициализация HTML-элемента
          const wrap = document.createElement("dd");

          // Запись текста ошибки
          samp.innerText = registry[error];

          // Запись HTML-элемента в список
          wrap.appendChild(samp);
          list.appendChild(wrap);

          // Реинициализация высоты
          height();
        }
      }

      // Реинициализация HTML-элемента с текстом ошибок
      if (
        reinitialization && list.childElementCount === 0
      ) {
        wrap.classList.add("hidden");
      } else {
        // УЛОВКА: таким образом не запускается анимация до взаимодействия с элементом (исправлял это в CSS, но не помню как)
        this.body.errors.classList.add("animation");
        wrap.classList.remove("hidden");
      }

      return list.childElementCount === 0 ? false : true;
    }

    /**
     * Сотрудник
     */
    static worker = class {
      /**
       * Ссылка на ядро (родительский класс)
       */
      static core;

      /**
       * Тело всплывающего окна (массив)
       */
      static body = {};

      /**
       * Сгенерировать всплывающее окно c данными сотрудника
       *
       * @param {HTMLElement} row Строка
       *
       * @return {void}
       */
      static async popup(row) {
        if (row instanceof HTMLElement) {
          // Получена строка

          // Инициализация идентификатора сотрудника
          const worker =
            row.querySelector('span[data-column="worker"]').innerText;

          // Инициализация идентификатора строки
          const task = row.getAttribute("id");

          // Запрос к серверу
          return await fetch(`/worker/${worker}/read`, {
            method: "POST",
            headers: {
              "Content-Type": "application/x-www-form-urlencoded",
            },
          })
            .then((response) => response.text())
            .then((data) => {
              if (data.length > 0) {
                // Получены данные (подразумевается, что сотрудник найден)

                // Инициализация буфера выбранных строк
                const buffer = [
                  ...document.querySelectorAll('div[data-selected="true"]'),
                ];

                // Удаление статуса активной строки у остальных строк
                for (const element of buffer) {
                  element.removeAttribute("data-selected");
                }

                // Инициализация статуса активной строки (обрабатываемой в данный момент)
                row.setAttribute("data-selected", "true");

                // Инициализация оболочки всплывающего окна
                this.body.wrap = document.createElement("div");
                this.body.wrap.setAttribute("id", "popup");

                // Инициализация всплывающего окна
                const popup = document.createElement("section");
                popup.classList.add("window", "list", "small");

                // Инициализация буфера для записи во всплывающее окно
                const list = document.createElement("div");

                // Инициализация оболочки для кнопок
                const buttons = document.createElement("div");
                buttons.classList.add("row", "divided", "buttons");

                // Инициализация оболочки для строки с полем ввода
                const label = document.createElement("label");

                // Инициализация иконки для поля ввода замены
                const icon = document.createElement("i");
                icon.classList.add("icon", "user");

                // Инициализация поля ввода замены
                const input = document.createElement("input");
                input.setAttribute("placeholder", "Сотрудник");
                input.setAttribute(
                  "onkeydown",
                  "if (event.keyCode === 13) this.nextElementSibling.click()",
                );
                input.setAttribute("autocomplete", "username");
                input.setAttribute("autofocus", "true");
                input.setAttribute("title", "Идентификатор сотрудника");
                input.setAttribute("list", "popup_worker_datalist");

                // Инициализация кнопки смены
                const replace = document.createElement("button");
                replace.classList.add("sea");
                replace.innerText = "Заменить";
                replace.setAttribute(
                  "onclick",
                  `tasks.worker.update(document.getElementById('${task}'), this.previousElementSibling, this)`,
                );

                // Инициализация списка сотружников
                const datalist = document.createElement("datalist");
                datalist.setAttribute("id", "popup_worker_datalist");
                workers.list().then((html) => datalist.innerHTML = html);

                // Инициализация кнопки удаления
                const remove = document.createElement("button");
                remove.classList.add("clay");
                remove.innerText = "Удалить";
                remove.setAttribute(
                  "onclick",
                  `tasks.worker.remove(document.getElementById('${task}'), this)`,
                );

                // Инициализация окна с ошибками
                this.body.errors = document.createElement("section");
                this.body.errors.classList.add(
                  "errors",
                  "window",
                  "list",
                  "small",
                  "hidden",
                );

                // Инициализация элемента-тела (оболочки) окна с ошибками
                const errors = document.createElement("section");
                errors.classList.add("body");

                // Инициализация элемента-списка ошибок
                const dl = document.createElement("dl");

                // Инициализация устаревшего всплывающего окна
                const old = document.getElementById("popup");

                // Удаление устаревшего всплывающего окна
                if (old instanceof HTMLElement) old.remove();

                // Запись в документ
                label.appendChild(icon);
                label.appendChild(input);
                label.appendChild(replace);
                label.appendChild(remove);
                label.appendChild(datalist);
                buttons.appendChild(label);
                popup.appendChild(list);
                popup.appendChild(buttons);
                this.body.wrap.appendChild(popup);
                errors.appendChild(dl);
                this.body.errors.appendChild(errors);
                this.body.wrap.appendChild(this.body.errors);
                document.body.appendChild(this.body.wrap);

                // Фокусировка
                input.focus();

                // Запись в документ HTML-данных через буфер
                list.outerHTML = data;

                // Инициализация элемента с номером в списке данных сотрудника
                const number = document.getElementById(`${worker}_number`)
                  .getElementsByTagName("a")[0];

                // Инициализация маски номера
                const mask = IMask.createMask({ mask: "+{7} (000) 000-00-00" });

                // Применение маски номера
                mask.resolve(number.innerText);

                // Запись нового значения
                number.innerText = mask.value;

                // Инициализация переменных для окна с ошибками (12 - это значение gap из div#popup)
                function top(errors) {
                  errors.style.setProperty(
                    "transition",
                    "0s",
                  );
                  errors.style.setProperty(
                    "--top",
                    popup.offsetTop + popup.offsetHeight + 12 + "px",
                  );
                  setTimeout(
                    () => errors.style.removeProperty("transition"),
                    100,
                  );
                }
                top(this.body.errors);
                const resize = new ResizeObserver(() => top(this.body.errors));
                resize.observe(this.body.wrap);

                // Инициалиация маски идентификатора сотрудника в поле ввода идентификатора сотрудника
                IMask(input, { mask: "000000" });

                // Инициализация функции закрытия всплывающего окна
                const click = () => {
                  // Блокировка выполнения
                  if (this.freeze) return;

                  // Удаление всплывающего окна
                  this.body.wrap.remove();

                  // Инициализация буфера выбранных строк
                  const buffer = [
                    ...document.querySelectorAll('div[data-selected="true"]'),
                  ];

                  // Удаление статуса активной строки у остальных строк
                  for (const element of buffer) {
                    element.removeAttribute("data-selected");
                  }
                };

                // Инициализация функции добавления функции закрытия всплывающего окна
                const enable = () =>
                  this.body.wrap.addEventListener("click", click);

                // Инициализация функции удаления функции закрытия всплывающего окна
                const disable = () =>
                  this.body.wrap.removeEventListener("click", click);

                // Первичная активация функции удаления всплывающего окна
                enable();

                // Добавление функций удаления всплывающего окна по событиям
                popup.addEventListener("mouseenter", disable);
                popup.addEventListener("mouseleave", enable);
              } else {
                // Не получены данные (подразумевается, что сотрудник не найден)

                // Инициализация буфера выбранных строк
                const buffer = [
                  ...document.querySelectorAll('div[data-selected="true"]'),
                ];

                // Удаление статуса активной строки у остальных строк
                for (const element of buffer) {
                  element.removeAttribute("data-selected");
                }

                // Инициализация статуса активной строки (обрабатываемой в данный момент)
                row.setAttribute("data-selected", "true");

                // Инициализация оболочки всплывающего окна
                this.body.wrap = document.createElement("div");
                this.body.wrap.setAttribute("id", "popup");

                // Инициализация всплывающего окна
                const popup = document.createElement("section");
                popup.classList.add("window", "list", "small");

                // Инициализация оболочки для кнопок
                const buttons = document.createElement("div");
                buttons.classList.add("row", "buttons");

                // Инициализация оболочки для строки с полем ввода
                const label = document.createElement("label");

                // Инициализация иконки для поля ввода замены
                const icon = document.createElement("i");
                icon.classList.add("icon", "user");

                // Инициализация поля ввода замены
                const input = document.createElement("input");
                input.setAttribute("placeholder", "Сотрудник");
                input.setAttribute(
                  "onkeydown",
                  "if (event.keyCode === 13) this.nextElementSibling.click()",
                );
                input.setAttribute("autocomplete", "username");
                input.setAttribute("autofocus", "true");
                input.setAttribute("title", "Идентификатор сотрудника");
                input.setAttribute("list", "popup_worker_datalist");

                // Инициализация кнопки замены
                const connect = document.createElement("button");
                connect.classList.add("grass");
                connect.innerText = "Назначить";
                connect.setAttribute(
                  "onclick",
                  `tasks.worker.update(document.getElementById('${task}'), this.previousElementSibling, this)`,
                );

                // Инициализация списка сотружников
                const datalist = document.createElement("datalist");
                datalist.setAttribute("id", "popup_worker_datalist");
                workers.list().then((html) => datalist.innerHTML = html);

                // Инициализация оболочки для кнопок
                const buttons_2 = document.createElement("div");
                buttons_2.classList.add("row", "buttons");

                // Инициализация оболочки для строки с полем ввода
                const label_2 = document.createElement("label");

                // Инициализация кнопки замены
                const publish = document.createElement("button");
                publish.classList.add("sea");
                publish.innerText = "Опубликовать для поиска";
                publish.setAttribute(
                  "onclick",
                  `tasks.worker.publish(document.getElementById('${task}'), this)`,
                );

                // Инициализация окна с ошибками
                this.body.errors = document.createElement("section");
                this.body.errors.classList.add(
                  "errors",
                  "window",
                  "list",
                  "small",
                  "hidden",
                );

                // Инициализация элемента-тела (оболочки) окна с ошибками
                const body = document.createElement("section");
                body.classList.add("body");

                // Инициализация элемента-списка ошибок
                const dl = document.createElement("dl");

                // Инициализация устаревшего всплывающего окна
                const old = document.getElementById("popup");

                // Удаление устаревшего всплывающего окна
                if (old instanceof HTMLElement) old.remove();

                // Запись в документ
                label.appendChild(icon);
                label.appendChild(input);
                label.appendChild(connect);
                label.appendChild(datalist);
                buttons.appendChild(label);
                popup.appendChild(buttons);
                label_2.appendChild(publish);
                buttons_2.appendChild(label_2);
                popup.appendChild(buttons_2);
                this.body.wrap.appendChild(popup);
                body.appendChild(dl);
                this.body.errors.appendChild(body);
                this.body.wrap.appendChild(this.body.errors);
                document.body.appendChild(this.body.wrap);

                // Инициализация переменных для окна с ошибками (12 - это значение gap из div#popup)
                function top(errors) {
                  errors.style.setProperty(
                    "transition",
                    "0s",
                  );
                  errors.style.setProperty(
                    "--top",
                    popup.offsetTop + popup.offsetHeight + 12 + "px",
                  );
                  setTimeout(
                    () => errors.style.removeProperty("transition"),
                    100,
                  );
                }
                top(this.body.errors);
                const resize = new ResizeObserver(() => top(this.body.errors));
                resize.observe(this.body.wrap);

                // Инициалиация маски идентификатора сотрудника
                IMask(input, { mask: "000000" });

                // Инициализация функции закрытия всплывающего окна
                const click = () => {
                  // Блокировка выполнения
                  if (this.freeze) return;

                  // Удаление всплывающего окна
                  this.body.wrap.remove();

                  // Инициализация буфера выбранных строк
                  const buffer = [
                    ...document.querySelectorAll('div[data-selected="true"]'),
                  ];

                  // Удаление статуса активной строки у остальных строк
                  for (const element of buffer) {
                    element.removeAttribute("data-selected");
                  }
                };

                // Инициализация функции добавления функции закрытия всплывающего окна
                const enable = () =>
                  this.body.wrap.addEventListener("click", click);

                // Инициализация функции удаления функции закрытия всплывающего окна
                const disable = () =>
                  this.body.wrap.removeEventListener("click", click);

                // Первичная активация функции удаления всплывающего окна
                enable();

                // Добавление функций удаления всплывающего окна по событиям
                popup.addEventListener("mouseenter", disable);
                popup.addEventListener("mouseleave", enable);
              }
            });
        }
      }

      /**
       * Обновить
       *
       * @param {HTMLElement} row Строка
       * @param {HTMLElement} input Поле для ввода идентификатора сотрудника
       * @param {HTMLElement} button Кнопка
       *
       * @return {void}
       */
      static update(row, input, button) {
        // Блокировка поля ввода
        input.setAttribute("readonly", true);
        button.setAttribute("disabled", true);

        // Деинициализация индикатора и анимации об ошибке
        input.classList.remove("error");
        input.previousElementSibling.classList.remove("error");

        // Сброс анимации
        this.body.errors.classList.add("hidden");

        // Запуск выполнения
        this._update(row, input, button);
      }

      /**
       * Обновить (системное)
       *
       * @param {HTMLElement} row Строка
       * @param {HTMLElement} input Поле для ввода идентификатора сотрудника
       * @param {HTMLElement} button Кнопка
       *
       * @return {void}
       */
      static _update = damper(async (row, input, button) => {
        if (
          row instanceof HTMLElement &&
          input instanceof HTMLElement &&
          button instanceof HTMLElement
        ) {
          // Получена строка, поле для ввода и кнопка

          // Инициализация идентификатора строки
          const id = row.getAttribute("id");

          if (typeof id === "string") {
            // Инициализирован идентификатор

            // Инициализация функции разблокировки
            function unblock() {
              // Разблокировка поля ввода
              input.removeAttribute("readonly");
              button.removeAttribute("disabled");
            }

            // Запуск отсрочки разблокировки на случай, если сервер не отвечает
            const timeout = setTimeout(() => {
              this.errors(["Сервер не отвечает"]);
              unblock();
            }, 5000);

            // Запрос к серверу
            return await fetch(`/task/${id}/worker/update`, {
              method: "POST",
              headers: {
                "Content-Type": "application/x-www-form-urlencoded",
              },
              body: `worker=${input.value}`,
            })
              .then((response) => response.json())
              .then((data) => {
                // Удаление отсрочки разблокировки
                clearTimeout(timeout);

                if (this.errors(data.errors)) {
                  // Сгенерированы ошибки

                  // Инициализация отображения ошибки
                  input.classList.add("error");
                  input.previousElementSibling.classList.add("error");

                  // Фокусировка на поле ввода
                  input.focus();

                  // Разблокировка полей ввода и кнопок
                  unblock();
                } else {
                  // Не сгенерированы ошибки (подразумевается их отсутствие)

                  if (data.updated) {
                    // Записано обновление

                    // Инициализация буффера списка классов
                    const buffer = [...row.classList];

                    // Реинициализация строки
                    row.outerHTML = data.row;

                    // Реинициализация строки (выражение странное, но правильное)
                    row = document.getElementById(row.getAttribute("id"));

                    // Копирование классов из буфера классов удалённой строки
                    row.classList.add(...buffer);
                  }

                  // Реинициализация всплывающего окна
                  if (this.body.wrap instanceof HTMLElement) {
                    tasks.worker.popup(row);
                  }
                }
              });
          }
        }
      }, 300);

      /**
       * Удалить
       *
       * @param {HTMLElement} row Строка
       * @param {HTMLElement} button Кнопка
       *
       * @return {void}
       */
      static remove(row, button) {
        // Запуск выполнения
        this._remove(row, button);
      }

      /**
       * Удалить (системное)
       *
       * @param {HTMLElement} row Строка
       * @param {HTMLElement} button Кнопка
       *
       * @return {void}
       */
      static _remove = damper(async (row, button) => {
        if (
          row instanceof HTMLElement &&
          button instanceof HTMLElement
        ) {
          // Получена строка и кнопка

          // Инициализация идентификатора строки
          const id = row.getAttribute("id");

          if (typeof id === "string") {
            // Инициализирован идентификатор

            // Запуск отсрочки разблокировки на случай, если сервер не отвечает
            const timeout = setTimeout(() => {
              this.errors(["Сервер не отвечает"]);
              unblock();
            }, 5000);

            // Запрос к серверу
            return await fetch(`/task/${id}/worker/update`, {
              method: "POST",
              headers: {
                "Content-Type": "application/x-www-form-urlencoded",
              },
              body: `worker=delete`,
            })
              .then((response) => response.json())
              .then((data) => {
                // Удаление отсрочки разблокировки
                clearTimeout(timeout);

                if (this.errors(data.errors)) {
                  // Сгенерированы ошибки

                  // Разблокировка полей ввода и кнопок
                  unblock();
                } else {
                  // Не сгенерированы ошибки (подразумевается их отсутствие)

                  if (data.updated) {
                    // Записано обновление

                    // Инициализация буффера списка классов
                    const buffer = [...row.classList];

                    // Реинициализация строки
                    row.outerHTML = data.row;

                    // Реинициализация строки (выражение странное, но правильное)
                    row = document.getElementById(row.getAttribute("id"));

                    // Копирование классов из буфера классов удалённой строки
                    row.classList.add(...buffer);
                  }

                  // Реинициализация всплывающего окна
                  if (this.body.wrap instanceof HTMLElement) {
                    tasks.worker.popup(row);
                  }
                }
              });
          }
        }
      }, 300);

      /**
       * Сгенерировать HTML-элемент со списком ошибок
       *
       * @param {object} registry Реестр ошибок
       * @param {bool} reinitialization Реинициализировать?
       *
       * @return {bool} Сгенерированы ошибки?
       */
      static errors(registry, reinitialization = true) {
        // Инициализация ссылки на HTML-элемент с ошибками
        const wrap = this.body.errors;

        // Перерасчёт высоты элемента
        function height() {
          wrap.classList.remove("hidden");
          wrap.classList.remove("animation");
          // Реинициализация переменной с данными о высоте HTML-элемента (16 - это padding-top + padding-bottom у div#popup > section.errors)
          wrap.style.setProperty(
            "--height",
            wrap.offsetHeight - 16 + "px",
          );
          wrap.classList.add("animation");
          wrap.classList.add("hidden");
        }

        // Инициализация элемента-списка ошибок
        const list = wrap.getElementsByTagName("dl")[0];

        // Удаление ошибок из прошлой генерации
        if (reinitialization) list.innerHTML = null;

        for (const error in registry) {
          // Генерация HTML-элементов с текстами ошибок

          // Инициализация HTML-элемента текста ошибки
          const samp = document.createElement("samp");

          if (typeof registry[error] === "object") {
            // Категория ошибок

            // Проверка наличия ошибок
            if (registry[error].length === 0) continue;

            // Инициализация HTML-элемента-оболочки
            const wrap = document.createElement("dt");

            // Запись текста категории
            samp.innerText = error;

            // Запись HTML-элементов в список
            wrap.appendChild(samp);
            list.appendChild(wrap);

            // Реинициализация высоты
            height();

            // Обработка вложенных ошибок (вход в рекурсию)
            this.errors(registry[error], false);
          } else {
            // Текст ошибки (подразумевается)

            // Инициализация HTML-элемента
            const wrap = document.createElement("dd");

            // Запись текста ошибки
            samp.innerText = registry[error];

            // Запись HTML-элемента в список
            wrap.appendChild(samp);
            list.appendChild(wrap);

            // Реинициализация высоты
            height();
          }
        }

        // Реинициализация HTML-элемента с текстом ошибок
        if (
          reinitialization && list.childElementCount === 0
        ) {
          wrap.classList.add("hidden");
        } else {
          // УЛОВКА: таким образом не запускается анимация до взаимодействия с элементом (исправлял это в CSS, но не помню как)
          this.body.errors.classList.add("animation");
          wrap.classList.remove("hidden");
        }

        return list.childElementCount === 0 ? false : true;
      }
    };

    /**
     * Магазин
     */
    static market = class {
      /**
       * Ссылка на ядро (родительский класс)
       */
      static core;

      /**
       * Тело всплывающего окна (массив)
       */
      static body = {};

      /**
       * Сгенерировать всплывающее окно c данными магазина
       *
       * @param {HTMLElement} row Строка
       *
       * @return {void}
       */
      static async popup(row) {
        if (row instanceof HTMLElement) {
          // Получена строка

          // Инициализация идентификатора магазина
          const market = "K" +
            row.querySelector('span[data-column="market"]').innerText.replace(
              /^[kк]/i,
              "",
            );

          // Инициализация идентификатора строки
          const task = row.getAttribute("id");

          // Запрос к серверу
          return await fetch(`/market/${market}/read`, {
            method: "POST",
            headers: {
              "Content-Type": "application/x-www-form-urlencoded",
            },
          })
            .then((response) => response.text())
            .then((data) => {
              if (data.length > 0) {
                // Получены данные (подразумевается, что магазин найден)

                // Инициализация буфера выбранных строк
                const buffer = [
                  ...document.querySelectorAll('div[data-selected="true"]'),
                ];

                // Удаление статуса активной строки у остальных строк
                for (const element of buffer) {
                  element.removeAttribute("data-selected");
                }

                // Инициализация статуса активной строки (обрабатываемой в данный момент)
                row.setAttribute("data-selected", "true");

                // Инициализация оболочки всплывающего окна
                this.body.wrap = document.createElement("div");
                this.body.wrap.setAttribute("id", "popup");

                // Инициализация всплывающего окна
                const popup = document.createElement("section");
                popup.classList.add("window", "list", "small");

                // Инициализация буфера для записи во всплывающее окно
                const list = document.createElement("div");

                // Инициализация оболочки для кнопок
                const buttons = document.createElement("div");
                buttons.classList.add("row", "buttons");

                // Инициализация оболочки для строки с полем ввода
                const label = document.createElement("label");

                // Инициализация иконки для поля ввода замены
                const icon = document.createElement("i");
                icon.classList.add("icon", "user");

                // Инициализация поля ввода замены
                const input = document.createElement("input");
                input.setAttribute("placeholder", "Магазин");
                input.setAttribute(
                  "onkeydown",
                  "if (event.keyCode === 13) this.nextElementSibling.click()",
                );
                input.setAttribute("autocomplete", "username");
                input.setAttribute("autofocus", "true");
                input.setAttribute("title", "Идентификатор магазина");
                input.setAttribute("list", "popup_market_datalist");

                // Инициализация кнопки смены
                const replace = document.createElement("button");
                replace.classList.add("sea");
                replace.innerText = "Заменить";
                replace.setAttribute(
                  "onclick",
                  `tasks.market.update(document.getElementById('${task}'), this.previousElementSibling, this)`,
                );

                // Инициализация списка сотружников
                const datalist = document.createElement("datalist");
                datalist.setAttribute("id", "popup_market_datalist");
                markets.list().then((html) => datalist.innerHTML = html);

                // Инициализация кнопки удаления
                const remove = document.createElement("button");
                remove.classList.add("clay");
                remove.innerText = "Удалить";
                remove.setAttribute(
                  "onclick",
                  `tasks.market.remove(document.getElementById('${task}'), this)`,
                );

                // Инициализация окна с ошибками
                this.body.errors = document.createElement("section");
                this.body.errors.classList.add(
                  "errors",
                  "window",
                  "list",
                  "small",
                  "hidden",
                );

                // Инициализация элемента-тела (оболочки) окна с ошибками
                const body = document.createElement("section");
                body.classList.add("body");

                // Инициализация элемента-списка ошибок
                const dl = document.createElement("dl");

                // Инициализация устаревшего всплывающего окна
                const old = document.getElementById("popup");

                // Удаление устаревшего всплывающего окна
                if (old instanceof HTMLElement) old.remove();

                // Запись в документ
                label.appendChild(icon);
                label.appendChild(input);
                label.appendChild(replace);
                label.appendChild(remove);
                label.appendChild(datalist);
                buttons.appendChild(label);
                popup.appendChild(list);
                popup.appendChild(buttons);
                this.body.wrap.appendChild(popup);
                body.appendChild(dl);
                this.body.errors.appendChild(body);
                document.body.appendChild(this.body.wrap);

                // Фокусировка
                input.focus();

                // Запись в документ HTML-данных через буфер
                list.outerHTML = data;

                // Инициализация переменных для окна с ошибками (12 - это значение gap из div#popup)
                function top(errors) {
                  errors.style.setProperty(
                    "transition",
                    "0s",
                  );
                  errors.style.setProperty(
                    "--top",
                    popup.offsetTop + popup.offsetHeight + 12 + "px",
                  );
                  setTimeout(
                    () => errors.style.removeProperty("transition"),
                    100,
                  );
                }
                top(this.body.errors);
                const resize = new ResizeObserver(() => top(this.body.errors));
                resize.observe(this.body.wrap);

                // Инициалиация маски идентификатора магазина в поле ввода идентификатора магазина
                IMask(input, { mask: "K000000" });

                // Инициализация функции закрытия всплывающего окна
                const click = () => {
                  // Блокировка выполнения
                  if (this.freeze) return;

                  // Удаление всплывающего окна
                  this.body.wrap.remove();

                  // Инициализация буфера выбранных строк
                  const buffer = [
                    ...document.querySelectorAll('div[data-selected="true"]'),
                  ];

                  // Удаление статуса активной строки у остальных строк
                  for (const element of buffer) {
                    element.removeAttribute("data-selected");
                  }
                };

                // Инициализация функции добавления функции закрытия всплывающего окна
                const enable = () =>
                  this.body.wrap.addEventListener("click", click);

                // Инициализация функции удаления функции закрытия всплывающего окна
                const disable = () =>
                  this.body.wrap.removeEventListener("click", click);

                // Первичная активация функции удаления всплывающего окна
                enable();

                // Добавление функций удаления всплывающего окна по событиям
                popup.addEventListener("mouseenter", disable);
                popup.addEventListener("mouseleave", enable);
              } else {
                // Не получены данные (подразумевается, что магазин не найден)

                // Инициализация буфера выбранных строк
                const buffer = [
                  ...document.querySelectorAll('div[data-selected="true"]'),
                ];

                // Удаление статуса активной строки у остальных строк
                for (const element of buffer) {
                  element.removeAttribute("data-selected");
                }

                // Инициализация статуса активной строки (Магазинобрабатываемой в данный момент)
                row.setAttribute("data-selected", "true");

                // Инициализация оболочки всплывающего окна
                this.body.wrap = document.createElement("div");
                this.body.wrap.setAttribute("id", "popup");

                // Инициализация всплывающего окна
                const popup = document.createElement("section");
                popup.classList.add("window", "list", "small");

                // Инициализация оболочки для кнопок
                const buttons = document.createElement("div");
                buttons.classList.add("row", "buttons");

                // Инициализация оболочки для строки с полем ввода
                const label = document.createElement("label");

                // Инициализация иконки для поля ввода замены
                const icon = document.createElement("i");
                icon.classList.add("icon", "user");

                // Инициализация поля ввода замены
                const input = document.createElement("input");
                input.setAttribute("placeholder", "Магазин");
                input.setAttribute(
                  "onkeydown",
                  "if (event.keyCode === 13) this.nextElementSibling.click()",
                );
                input.setAttribute("autocomplete", "username");
                input.setAttribute("autofocus", "true");
                input.setAttribute("title", "Идентификатор магазина");
                input.setAttribute("list", "popup_market_datalist");

                // Инициализация кнопки замены
                const connect = document.createElement("button");
                connect.classList.add("grass");
                connect.innerText = "Назначить";
                connect.setAttribute(
                  "onclick",
                  `tasks.market.update(document.getElementById('${task}'), this.previousElementSibling, this)`,
                );

                // Инициализация списка сотружников
                const datalist = document.createElement("datalist");
                datalist.setAttribute("id", "popup_market_datalist");
                markets.list().then((html) => datalist.innerHTML = html);

                // Инициализация окна с ошибками
                this.body.errors = document.createElement("section");
                this.body.errors.classList.add(
                  "errors",
                  "window",
                  "list",
                  "small",
                  "hidden",
                );

                // Инициализация элемента-тела (оболочки) окна с ошибками
                const body = document.createElement("section");
                body.classList.add("body");

                // Инициализация элемента-списка ошибок
                const dl = document.createElement("dl");

                // Инициализация устаревшего всплывающего окна
                const old = document.getElementById("popup");

                // Удаление устаревшего всплывающего окна
                if (old instanceof HTMLElement) old.remove();

                // Запись в документ
                label.appendChild(icon);
                label.appendChild(input);
                label.appendChild(connect);
                label.appendChild(datalist);
                buttons.appendChild(label);
                popup.appendChild(buttons);
                this.body.wrap.appendChild(popup);
                body.appendChild(dl);
                this.body.errors.appendChild(body);
                this.body.wrap.appendChild(this.body.errors);
                document.body.appendChild(this.body.wrap);

                // Инициализация переменных для окна с ошибками (12 - это значение gap из div#popup)
                function top(errors) {
                  errors.style.setProperty(
                    "transition",
                    "0s",
                  );
                  errors.style.setProperty(
                    "--top",
                    popup.offsetTop + popup.offsetHeight + 12 + "px",
                  );
                  setTimeout(
                    () => errors.style.removeProperty("transition"),
                    100,
                  );
                }
                top(this.body.errors);
                const resize = new ResizeObserver(() => top(this.body.errors));
                resize.observe(this.body.wrap);

                // Инициалиация маски идентификатора магазина
                IMask(input, { mask: "K000000" });

                // Инициализация функции закрытия всплывающего окна
                const click = () => {
                  // Блокировка выполнения
                  if (this.freeze) return;

                  // Удаление всплывающего окна
                  this.body.wrap.remove();

                  // Инициализация буфера выбранных строк
                  const buffer = [
                    ...document.querySelectorAll('div[data-selected="true"]'),
                  ];

                  // Удаление статуса активной строки у остальных строк
                  for (const element of buffer) {
                    element.removeAttribute("data-selected");
                  }
                };

                // Инициализация функции добавления функции закрытия всплывающего окна
                const enable = () =>
                  this.body.wrap.addEventListener("click", click);

                // Инициализация функции удаления функции закрытия всплывающего окна
                const disable = () =>
                  this.body.wrap.removeEventListener("click", click);

                // Первичная активация функции удаления всплывающего окна
                enable();

                // Добавление функций удаления всплывающего окна по событиям
                popup.addEventListener("mouseenter", disable);
                popup.addEventListener("mouseleave", enable);
              }
            });
        }
      }

      /**
       * Обновить
       *
       * @param {HTMLElement} row Строка
       * @param {HTMLElement} input Поле для ввода идентификатора магазина
       * @param {HTMLElement} button Кнопка
       *
       * @return {void}
       */
      static update(row, input, button) {
        // Блокировка поля ввода
        input.setAttribute("readonly", true);
        button.setAttribute("disabled", true);

        // Деинициализация индикатора и анимации об ошибке
        input.classList.remove("error");
        input.previousElementSibling.classList.remove("error");

        // Сброс анимации
        this.body.errors.classList.add("hidden");

        // Запуск выполнения
        this._update(row, input, button);
      }

      /**
       * Обновить (системное)
       *
       * @param {HTMLElement} row Строка
       * @param {HTMLElement} input Поле для ввода идентификатора магазина
       * @param {HTMLElement} button Кнопка
       *
       * @return {void}
       */
      static _update = damper(async (row, input, button) => {
        if (
          row instanceof HTMLElement &&
          input instanceof HTMLElement &&
          button instanceof HTMLElement
        ) {
          // Получена строка, поле для ввода и кнопка

          // Инициализация идентификатора строки
          const id = row.getAttribute("id");

          if (typeof id === "string") {
            // Инициализирован идентификатор

            // Инициализация функции разблокировки
            function unblock() {
              // Разблокировка поля ввода
              input.removeAttribute("readonly");
              button.removeAttribute("disabled");
            }

            // Запуск отсрочки разблокировки на случай, если сервер не отвечает
            const timeout = setTimeout(() => {
              this.errors(["Сервер не отвечает"]);
              unblock();
            }, 5000);

            // Запрос к серверу
            return await fetch(`/task/${id}/market/update`, {
              method: "POST",
              headers: {
                "Content-Type": "application/x-www-form-urlencoded",
              },
              body: `market=${input.value}`,
            })
              .then((response) => response.json())
              .then((data) => {
                // Удаление отсрочки разблокировки
                clearTimeout(timeout);

                if (this.errors(data.errors)) {
                  // Сгенерированы ошибки

                  // Инициализация отображения ошибки
                  input.classList.add("error");
                  input.previousElementSibling.classList.add("error");

                  // Фокусировка на поле ввода
                  input.focus();

                  // Разблокировка полей ввода и кнопок
                  unblock();
                } else {
                  // Не сгенерированы ошибки (подразумевается их отсутствие)

                  if (data.updated) {
                    // Записано обновление

                    // Инициализация буффера списка классов
                    const buffer = [...row.classList];

                    // Реинициализация строки
                    row.outerHTML = data.row;

                    // Реинициализация строки (выражение странное, но правильное)
                    row = document.getElementById(row.getAttribute("id"));

                    // Копирование классов из буфера классов удалённой строки
                    row.classList.add(...buffer);
                  }

                  // Реинициализация всплывающего окна
                  if (this.body.wrap instanceof HTMLElement) {
                    tasks.market.popup(row);
                  }
                }
              });
          }
        }
      }, 300);

      /**
       * Удалить
       *
       * @param {HTMLElement} row Строка
       * @param {HTMLElement} button Кнопка
       *
       * @return {void}
       */
      static remove(row, button) {
        console.log("bebre", this);
        // Запуск выполнения
        this._remove(row, button);
      }

      /**
       * Удалить (системное)
       *
       * @param {HTMLElement} row Строка
       * @param {HTMLElement} button Кнопка
       *
       * @return {void}
       */
      static _remove = damper(async (row, button) => {
        if (row instanceof HTMLElement && button instanceof HTMLElement) {
          // Получена строка и кнопка

          // Инициализация идентификатора строки
          const id = row.getAttribute("id");

          if (typeof id === "string") {
            // Инициализирован идентификатор

            // Запуск отсрочки разблокировки на случай, если сервер не отвечает
            const timeout = setTimeout(() => {
              this.errors(["Сервер не отвечает"]);
              unblock();
            }, 5000);

            // Запрос к серверу
            return await fetch(`/task/${id}/market/update`, {
              method: "POST",
              headers: {
                "Content-Type": "application/x-www-form-urlencoded",
              },
              body: `market=delete`,
            })
              .then((response) => response.json())
              .then((data) => {
                // Удаление отсрочки разблокировки
                clearTimeout(timeout);

                if (this.errors(data.errors)) {
                  // Сгенерированы ошибки

                  // Разблокировка полей ввода и кнопок
                  unblock();
                } else {
                  // Не сгенерированы ошибки (подразумевается их отсутствие)

                  if (data.updated) {
                    // Записано обновление

                    // Инициализация буффера списка классов
                    const buffer = [...row.classList];

                    // Реинициализация строки
                    row.outerHTML = data.row;

                    // Реинициализация строки (выражение странное, но правильное)
                    row = document.getElementById(row.getAttribute("id"));

                    // Копирование классов из буфера классов удалённой строки
                    row.classList.add(...buffer);
                  }

                  // Реинициализация всплывающего окна
                  if (this.body.wrap instanceof HTMLElement) {
                    tasks.market.popup(row);
                  }
                }
              });
          }
        }
      }, 300);

      /**
       * Сгенерировать HTML-элемент со списком ошибок
       *
       * @param {object} registry Реестр ошибок
       * @param {bool} reinitialization Реинициализировать?
       *
       * @return {bool} Сгенерированы ошибки?
       */
      static errors(registry, reinitialization = true) {
        // Инициализация ссылки на HTML-элемент с ошибками
        const wrap = this.body.errors;

        // Перерасчёт высоты элемента
        function height() {
          wrap.classList.remove("hidden");
          wrap.classList.remove("animation");
          // Реинициализация переменной с данными о высоте HTML-элемента (16 - это padding-top + padding-bottom у div#popup > section.errors)
          wrap.style.setProperty(
            "--height",
            wrap.offsetHeight - 16 + "px",
          );
          wrap.classList.add("animation");
          wrap.classList.add("hidden");
        }

        // Инициализация элемента-списка ошибок
        const list = wrap.getElementsByTagName("dl")[0];

        // Удаление ошибок из прошлой генерации
        if (reinitialization) list.innerHTML = null;

        for (const error in registry) {
          // Генерация HTML-элементов с текстами ошибок

          // Инициализация HTML-элемента текста ошибки
          const samp = document.createElement("samp");

          if (typeof registry[error] === "object") {
            // Категория ошибок

            // Проверка наличия ошибок
            if (registry[error].length === 0) continue;

            // Инициализация HTML-элемента-оболочки
            const wrap = document.createElement("dt");

            // Запись текста категории
            samp.innerText = error;

            // Запись HTML-элементов в список
            wrap.appendChild(samp);
            list.appendChild(wrap);

            // Реинициализация высоты
            height();

            // Обработка вложенных ошибок (вход в рекурсию)
            this.errors(registry[error], false);
          } else {
            // Текст ошибки (подразумевается)

            // Инициализация HTML-элемента
            const wrap = document.createElement("dd");

            // Запись текста ошибки
            samp.innerText = registry[error];

            // Запись HTML-элемента в список
            wrap.appendChild(samp);
            list.appendChild(wrap);

            // Реинициализация высоты
            height();
          }
        }

        // Реинициализация HTML-элемента с текстом ошибок
        if (
          reinitialization && list.childElementCount === 0
        ) {
          wrap.classList.add("hidden");
        } else {
          // УЛОВКА: таким образом не запускается анимация до взаимодействия с элементом (исправлял это в CSS, но не помню как)
          this.body.errors.classList.add("animation");
          wrap.classList.remove("hidden");
        }

        return list.childElementCount === 0 ? false : true;
      }
    };

    /**
     * Комментарий
     */
    static commentary = class {
      /**
       * Ссылка на ядро (родительский класс)
       */
      static core;

      /**
       * Тело всплывающего окна (массив)
       */
      static body = {};

      /**
       * Сгенерировать всплывающее окно c комментарием задачи
       *
       * @param {HTMLElement} row Строка
       *
       * @return {void}
       */
      static popup = damper((row) => {
        if (row instanceof HTMLElement) {
          // Получена строка

          // Инициализация идентификатора сотрудника
          const task = row.getAttribute("id");

          // Инициализация буфера выбранных строк
          const buffer = [
            ...document.querySelectorAll('div[data-selected="true"]'),
          ];

          // Удаление статуса активной строки у остальных строк
          for (const element of buffer) {
            element.removeAttribute("data-selected");
          }

          // Инициализация статуса активной строки (обрабатываемой в данный момент)
          row.setAttribute("data-selected", "true");

          // Инициализация оболочки всплывающего окна
          this.body.wrap = document.createElement("div");
          this.body.wrap.setAttribute("id", "popup");

          // Инициализация всплывающего окна
          const popup = document.createElement("section");
          popup.classList.add("list", "small");

          // Инициализация заголовка всплывающего окна
          const title = document.createElement("h3");
          title.innerText = "Комментарий";

          // Инициализация строки
          const row_1 = document.createElement("div");
          row_1.classList.add("row", "stretchable");

          // Инициализация оболочки для поля ввода комментария
          const label = document.createElement("label");

          // Инициализация поля ввода комментария
          const textarea = document.createElement("textarea");
          this.core.value(row, "commentary").then((text) =>
            textarea.value = text
          );
          textarea.setAttribute("autofocus", "true");
          textarea.setAttribute("maxlength", "800");
          textarea.setAttribute(
            "onkeyup",
            `tasks.commentary.update(document.getElementById('${task}'), this)`,
          );
          textarea.setAttribute("title", "Дополнительная информация");
          textarea.setAttribute(
            "placeholder",
            "Дополнительная информация о заявке",
          );

          // Инициализация окна с ошибками
          this.body.errors = document.createElement("section");
          this.body.errors.classList.add(
            "errors",
            "window",
            "list",
            "small",
            "hidden",
          );

          // Инициализация элемента-тела (оболочки) окна с ошибками
          const errors = document.createElement("section");
          errors.classList.add("body");

          // Инициализация элемента-списка ошибок
          const dl = document.createElement("dl");

          // Инициализация устаревшего всплывающего окна
          const old = document.getElementById("popup");

          // Удаление устаревшего всплывающего окна
          if (old instanceof HTMLElement) old.remove();

          // Удаление блокировки выполнения закрытия всплывающего окна
          this.freeze = false;

          // Запись в документ
          popup.appendChild(title);
          label.appendChild(textarea);
          row_1.appendChild(label);
          popup.appendChild(row_1);
          this.body.wrap.appendChild(popup);
          errors.appendChild(dl);
          this.body.errors.appendChild(errors);
          this.body.wrap.appendChild(this.body.errors);
          document.body.appendChild(this.body.wrap);

          // Фокусировка
          textarea.focus();

          // Инициализация переменных для окна с ошибками (12 - это значение gap из div#popup)
          function top(errors) {
            errors.style.setProperty(
              "transition",
              "0s",
            );
            errors.style.setProperty(
              "--top",
              popup.offsetTop + popup.offsetHeight + 12 + "px",
            );
            setTimeout(
              () => errors.style.removeProperty("transition"),
              100,
            );
          }
          top(this.body.errors);
          const resize = new ResizeObserver(() => top(this.body.errors));
          resize.observe(this.body.wrap);

          // Инициализация функции закрытия всплывающего окна
          const click = () => {
            // Удаление всплывающего окна
            this.body.wrap.remove();

            // Инициализация буфера выбранных строк
            const buffer = [
              ...document.querySelectorAll('div[data-selected="true"]'),
            ];

            // Удаление статуса активной строки у остальных строк
            for (const element of buffer) {
              element.removeAttribute("data-selected");
            }
          };

          // Инициализация функции добавления функции закрытия всплывающего окна
          const enable = () => this.body.wrap.addEventListener("click", click);

          // Инициализация функции удаления функции закрытия всплывающего окна
          const disable = () =>
            this.body.wrap.removeEventListener("click", click);

          // Первичная активация функции удаления всплывающего окна
          enable();

          // Добавление функций удаления всплывающего окна по событиям
          popup.addEventListener("mouseenter", disable);
          popup.addEventListener("mouseleave", enable);
        }
      }, 300);

      /**
       * Записать обновление
       *
       * @param {HTMLElement} row Строка
       * @param {HTMLElement} textarea HTML-элемент <textarea>
       *
       * @return {void}
       */
      static update(row, textarea) {
        // Сброс анимации ошибки
        textarea.classList.remove("error");

        // Запуск выполнения
        this._update(row, textarea);
      }

      /**
       * Записать обновление (системное)
       *
       * @param {HTMLElement} row Строка
       * @param {HTMLElement} textarea HTML-элемент <textarea>
       *
       * @return {void}
       */
      static _update = damper(async (row, textarea) => {
        if (row instanceof HTMLElement && textarea instanceof HTMLElement) {
          // Получена строка и select-элемент

          // Инициализация идентификатора строки
          const id = row.getAttribute("id");

          if (typeof id === "string") {
            // Инициализирован идентификатор

            // Запрос к серверу
            return await fetch(`/task/${id}/commentary`, {
              method: "POST",
              headers: {
                "Content-Type": "application/x-www-form-urlencoded",
              },
              body: `commentary=${textarea.value}`,
            })
              .then((response) => response.json())
              .then((data) => {
                if (data.writed) {
                  // Записано новое значение в базу данных

                  // Удаление анимации ошибки
                  textarea.classList.remove("error");

                  // Инициализация буффера списка классов
                  const buffer = [...row.classList];

                  // Реинициализация строки
                  row.outerHTML = data.row;

                  // Реинициализация строки (выражение странное, но правильное)
                  row = document.getElementById(row.getAttribute("id"));

                  // Копирование классов из буфера классов удалённой строки
                  row.classList.add(...buffer);
                } else {
                  // Не записано новое значение в базу данных

                  // Запись анимации ошибки
                  textarea.classList.add("error");
                }
              });
          }
        }
      }, 300);
    };
  };

  // Инициализация ссылок на ядро
  window.tasks.worker.core =
    window.tasks.market.core =
    window.tasks.commentary.core =
      window.tasks;
}

// Вызов события: "инициализировано"
document.dispatchEvent(
  new CustomEvent("tasks.initialized", {
    detail: { tasks: window.tasks },
  }),
);
