"use strict";

if (typeof window.workers !== "function") {
  // Not initialized

  // Initialize of the class in global namespace
  window.workers = class workers {
    /**
     * Заблокировать функцию закрытия всплывающего окна?
     */
    static freeze = false;

    /**
     * Тело всплывающего окна (массив)
     */
    static body = {};

    /**
     * Инициализирован класс?
     */
    static initialized = false;

    /**
     * Ожидание зависимости: ядро
     *
     * @param {function} execute Функция, которая будет встроена в демпфер
     * @param {mixed} args Аргументы встраиваемой функции
     *
     * @return {void}
     */
    static core(execute, ...args) {
      if (typeof execute === "function") {
        // Получена функция

        // Инициализация интервала для ожидания загрузки зависимостей
        const interval = setInterval(() => {
          if (typeof core === "function") {
            // Инициализировано ядро

            // Деинициализация интервала для ожидания загрузки записимостей
            clearInterval(interval);

            // Запуск выполнения
            return execute(...args);
          }
        }, 100);

        // Инициализация деинициализации интервала для ожидания загрузки зависимостей спустя большой срок времени
        setTimeout(() => clearInterval(interval), 3000);
      }
    }

    /**
     * Ожидание зависимости: демпфер
     *
     * @param {function} execute Функция, которая будет встроена в демпфер
     * @param {number} timer Количество миллисекунд для демпфера
     * @param {mixed} args Аргументы встраиваемой функции
     *
     * @return {void}
     */
    static damper(execute, timer = 3000, ...args) {
      if (typeof execute === "function") {
        // Получена функция

        // Инициализация интервала для ожидания загрузки зависимостей
        const interval = setInterval(() => {
          if (typeof damper === "function") {
            // Инициализирован демпфер

            // Деинициализация интервала для ожидания загрузки записимостей
            clearInterval(interval);

            // Запуск выполнения
            return damper(() => execute(...args), timer);
          }
        }, 100);

        // Инициализация деинициализации интервала для ожидания загрузки зависимостей спустя большой срок времени
        setTimeout(() => clearInterval(interval), 3000);
      }
    }

    /**
     * Авторизация
     *
     * @param {function} execute Функция, которая будет встроена в демпфер
     * @param {mixed} args Аргументы встраиваемой функции
     *
     * @return {void}
     */
    static authorization(execute, ...args) {
      if (typeof execute === "function") {
        // Получена функция

        if (core.page === this.page) {
          // Пройдена проверка на страницу

          // Запуск выполнения
          return execute(...args);
        }
      }
    }

    /**
     * Создать заявку
     *
     * @param {HTMLElement} cashiers Количество кассиров <input>
     * @param {HTMLElement} displayers Количество выкладчиков <input>
     * @param {HTMLElement} loaders Количество грузчиков <input>
     * @param {HTMLElement} gastronomes Количество гастрономов <input>
     * @param {HTMLElement} start Начало работы (00:00) <input>
     * @param {HTMLElement} end Конец работы (00:00) <input>
     * @param {HTMLElement} date Дата работы (d.m.Y) <input>
     * @param {HTMLElement} button Кнопка создания заявки <button>
     *
     * @return {void}
     */
    static _create(
      cashiers,
      displayers,
      loaders,
      gastronomes,
      start,
      end,
      date,
      button,
    ) {
      // Блокировка полей ввода
      cashiers.setAttribute("readonly", true);
      displayers.setAttribute("readonly", true);
      loaders.setAttribute("readonly", true);
      gastronomes.setAttribute("readonly", true);
      start.setAttribute("readonly", true);
      end.setAttribute("readonly", true);
      date.setAttribute("readonly", true);

      // Блокировка кнопки создания
      button.setAttribute("disabled", true);

      // Запуск выполнения
      this.__create(
        cashiers,
        displayers,
        loaders,
        gastronomes,
        start,
        end,
        date,
        button,
      );
    }

    /**
     * Создать заявку
     *
     * @param {HTMLElement} cashiers Количество кассиров <input>
     * @param {HTMLElement} displayers Количество выкладчиков <input>
     * @param {HTMLElement} loaders Количество грузчиков <input>
     * @param {HTMLElement} gastronomes Количество гастрономов <input>
     * @param {HTMLElement} start Начало работы (00:00) <input>
     * @param {HTMLElement} end Конец работы (00:00) <input>
     * @param {HTMLElement} date Дата работы (d.m.Y) <input>
     * @param {HTMLElement} button Кнопка создания заявки <button>
     *
     * @return {void}
     */
    static __create = damper(
      async (
        cashiers,
        displayers,
        loaders,
        gastronomes,
        start,
        end,
        date,
        button,
      ) => {
        // Инициализация функции разблокировки
        function unblock() {
          // Разблокировка полей ввода
          cashiers.removeAttribute("readonly");
          displayers.removeAttribute("readonly");
          loaders.removeAttribute("readonly");
          gastronomes.removeAttribute("readonly");
          start.removeAttribute("readonly");
          end.removeAttribute("readonly");
          date.removeAttribute("readonly");

          // Разблокировка кнопки
          button.removeAttribute("disabled");
        }

        // Запуск отсрочки разблокировки на случай, если сервер не отвечает
        const timeout = setTimeout(() => {
          this.errors(["Сервер не отвечает"]);
          unblock();
        }, 5000);

        // Запрос к серверу
        return await fetch("/tasks/create", {
          method: "POST",
          headers: {
            "Content-Type": "application/x-www-form-urlencoded",
          },
          body:
            `cashiers=${cashiers.value}&displayers=${displayers.value}&loaders=${loaders.value}&gastronomes=${gastronomes.value}&start=${start.value}&end=${end.value}&date=${
              date.valueAsDate / 1000
            }`,
        })
          .then((response) => response.json())
          .then((data) => {
            // Удаление отсрочки разблокировки
            clearTimeout(timeout);

            if (this.errors(data.errors)) {
              // Сгенерированы ошибки

              // Разблокировка полей ввода и кнопок
              unblock();
            } else {
              // Не сгенерированы ошибки (подразумевается их отсутствие)

              // Деинициализация всплывающего окна
              if (this.body.wrap instanceof HTMLElement) {
                this.body.wrap.remove();
              }

              // Реинициализация строк
              this.reinit();
            }
          });
      },
      300,
    );

    /**
     * @param {HTMLElement} row Строка
     *
     * @return {void}
     */
    static create = damper(() => {
      // Инициализация оболочки всплывающего окна
      this.body.wrap = document.createElement("div");
      this.body.wrap.setAttribute("id", "popup");

      // Инициализация всплывающего окна
      const popup = document.createElement("section");
      popup.classList.add("list", "small");

      // Инициализация заголовка всплывающего окна
      const title = document.createElement("h3");
      title.innerText = "Создание сотрудника";

      // Инициализация строки
      const row_name = document.createElement("div");
      row_name.classList.add("row", "merged");

      // Инициализация оболочки для строки
      const name = document.createElement("label");
      name.setAttribute("id", "name");

      // Инициализация заголовка для поля ввода
      const name_title = document.createElement("b");
      name_title.classList.add("separated", "right");
      name_title.innerText = "Имя:";

      // Инициализация поля ввода
      const name_input = document.createElement("input");
      name_input.classList.add("medium");
      name_input.setAttribute("type", "text");
      name_input.setAttribute("minlength", "0");
      name_input.setAttribute("maxlength", "100");
      name_input.setAttribute(
        "onkeydown",
        "if (event.keyCode === 13) this.parentElement.parentElement.nextElementSibling.nextElementSibling.nextElementSibling.nextElementSibling.children[0].children[0].click()",
      );
      name_input.setAttribute("title", "Имя сотрудника");
      name_input.setAttribute("placeholder", "Иван");

      // Инициализация строки
      const row_name_2 = document.createElement("div");
      row_name_2.classList.add("row", "merged");

      // Инициализация оболочки для строки
      const name_2 = document.createElement("label");
      name_2.setAttribute("id", "name_2");

      // Инициализация заголовка для поля ввода
      const name_2_title = document.createElement("b");
      name_2_title.classList.add("separated", "right");
      name_2_title.innerText = "Фамилия:";

      // Инициализация поля ввода
      const name_2_input = document.createElement("input");
      name_2_input.classList.add("medium");
      name_2_input.setAttribute("type", "text");
      name_2_input.setAttribute("minlength", "0");
      name_2_input.setAttribute("maxlength", "100");
      name_2_input.setAttribute(
        "onkeydown",
        "if (event.keyCode === 13) this.parentElement.parentElement.nextElementSibling.nextElementSibling.nextElementSibling.nextElementSibling.children[0].children[0].click()",
      );
      name_2_input.setAttribute("title", "Фамилия сотрудника");
      name_2_input.setAttribute("placeholder", "Иванов");

      // Инициализация строки
      const row_name_3 = document.createElement("div");
      row_name_3.classList.add("row", "merged");

      // Инициализация оболочки для строки
      const name_3 = document.createElement("label");
      name_3.setAttribute("id", "name_3");

      // Инициализация заголовка для поля ввода
      const name_3_title = document.createElement("b");
      name_3_title.classList.add("separated", "right");
      name_3_title.innerText = "Отчество:";

      // Инициализация поля ввода
      const name_3_input = document.createElement("input");
      name_3_input.classList.add("medium");
      name_3_input.setAttribute("type", "text");
      name_3_input.setAttribute("minlength", "0");
      name_3_input.setAttribute("maxlength", "100");
      name_3_input.setAttribute(
        "onkeydown",
        "if (event.keyCode === 13) this.parentElement.parentElement.nextElementSibling.nextElementSibling.nextElementSibling.nextElementSibling.children[0].children[0].click()",
      );
      name_3_input.setAttribute("title", "Отчество сотрудника");
      name_3_input.setAttribute("placeholder", "Иванович");

      // Инициализация строки
      const row_birth = document.createElement("div");
      row_birth.classList.add("row", "divided");

      // Инициализация оболочки для строки
      const birth = document.createElement("label");
      birth.setAttribute("id", "name");

      // Инициализация заголовка для поля ввода
      const birth_title = document.createElement("b");
      birth_title.classList.add("separated", "right");
      birth_title.innerText = "Дата рождения:";

      // Инициализация поля ввода
      const birth_input = document.createElement("input");
      birth_input.setAttribute("type", "date");
      birth_input.classList.add("medium");
      birth_input.setAttribute(
        "onkeydown",
        "if (event.keyCode === 13) this.nextElementSibling.click()",
      );
      birth_input.setAttribute("title", "Дата рождения сотрудника");
      birth_input.value = "1990.01.01";

      // Инициализация строки
      const row_number = document.createElement("div");
      row_number.classList.add("row");

      // Инициализация оболочки для строки
      const number = document.createElement("label");
      number.setAttribute("id", "name");

      // Инициализация заголовка для поля ввода
      const number_title = document.createElement("b");
      number_title.classList.add("separated", "right");
      number_title.innerText = "Номер:";

      // Инициализация поля ввода
      const number_input = document.createElement("input");
      number_input.classList.add("medium");
      number_input.setAttribute("type", "text");
      number_input.setAttribute("minlength", "18");
      number_input.setAttribute("maxlength", "18");
      number_input.setAttribute(
        "onkeydown",
        "if (event.keyCode === 13) this.parentElement.parentElement.nextElementSibling.nextElementSibling.nextElementSibling.nextElementSibling.children[0].children[0].click()",
      );
      number_input.setAttribute("title", "Номер SIM сотрудника");
      number_input.setAttribute("placeholder", "+7 (000) 000-00-00");

      // Инициализация строки
      const row_passport = document.createElement("div");
      row_passport.classList.add("row", "divided", "merged");

      // Инициализация оболочки для строки
      const passport = document.createElement("label");
      passport.setAttribute("id", "passport");

      // Инициализация заголовка для поля ввода
      const passport_title = document.createElement("b");
      passport_title.classList.add("separated", "right");
      passport_title.innerText = "Паспорт:";

      // Инициализация поля ввода
      const passport_input = document.createElement("input");
      passport_input.classList.add("medium");
      passport_input.setAttribute("type", "text");
      passport_input.setAttribute("minlength", "0");
      passport_input.setAttribute("maxlength", "11");
      passport_input.setAttribute(
        "onkeydown",
        "if (event.keyCode === 13) this.parentElement.parentElement.nextElementSibling.nextElementSibling.nextElementSibling.nextElementSibling.children[0].children[0].click()",
      );
      passport_input.setAttribute("title", "Паспортный номер сотрудника");
      passport_input.setAttribute("placeholder", "0000 000000");

      // Инициализация строки
      const row_issued = document.createElement("div");
      row_issued.classList.add("row", "merged");

      // Инициализация оболочки для строки
      const issued = document.createElement("label");
      issued.setAttribute("id", "issued");

      // Инициализация заголовка для поля ввода
      const issued_title = document.createElement("b");
      issued_title.classList.add("separated", "right");
      issued_title.innerText = "Дата выдачи:";

      // Инициализация поля ввода
      const issued_input = document.createElement("input");
      issued_input.setAttribute("type", "date");
      issued_input.classList.add("medium");
      issued_input.setAttribute(
        "onkeydown",
        "if (event.keyCode === 13) this.nextElementSibling.click()",
      );
      issued_input.setAttribute("title", "Дата выдачи паспорта сотрудника");
      issued_input.value = "2010.01.01";

      // Инициализация строки
      const row_department = document.createElement("div");
      row_department.classList.add("row", "merged");

      // Инициализация оболочки для строки
      const department = document.createElement("label");
      department.setAttribute("id", "department");

      // Инициализация заголовка для поля ввода
      const department_title = document.createElement("b");
      department_title.classList.add("separated", "right");
      department_title.innerText = "Отделение:";

      // Инициализация поля ввода
      const department_input = document.createElement("input");
      department_input.classList.add("medium");
      department_input.setAttribute("type", "text");
      department_input.setAttribute("minlength", "0");
      department_input.setAttribute("maxlength", "7");
      department_input.setAttribute(
        "onkeydown",
        "if (event.keyCode === 13) this.parentElement.parentElement.nextElementSibling.nextElementSibling.nextElementSibling.nextElementSibling.children[0].children[0].click()",
      );
      department_input.setAttribute(
        "title",
        "Номер отделения выдавшего паспорт сотруднику",
      );
      department_input.setAttribute("placeholder", "000-000");

      // Инициализация строки
      const row_department_2 = document.createElement("div");
      row_department_2.classList.add("row", "merged");

      // Инициализация оболочки для строки
      const department_2 = document.createElement("label");
      department_2.setAttribute("id", "department_2");

      // Инициализация заголовка для поля ввода
      const department_2_title = document.createElement("b");
      department_2_title.classList.add("separated", "right");
      department_2_title.innerText = "Адрес отделения:";

      // Инициализация поля ввода
      const department_2_input = document.createElement("input");
      department_2_input.classList.add("medium");
      department_2_input.setAttribute("type", "text");
      department_2_input.setAttribute("minlength", "0");
      department_2_input.setAttribute("maxlength", "150");
      department_2_input.setAttribute(
        "onkeydown",
        "if (event.keyCode === 13) this.parentElement.parentElement.nextElementSibling.nextElementSibling.nextElementSibling.nextElementSibling.children[0].children[0].click()",
      );
      department_2_input.setAttribute(
        "title",
        "Адрес отделения выдавшего паспорт сотруднику",
      );
      department_2_input.setAttribute("placeholder", "ОФМС России по ...");

      // Инициализация строки
      const row_requisites = document.createElement("div");
      row_requisites.classList.add("row", "divided", "merged");

      // Инициализация оболочки для строки
      const requisites = document.createElement("label");
      requisites.setAttribute("id", "requisites");

      // Инициализация заголовка для поля ввода
      const requisites_title = document.createElement("b");
      requisites_title.classList.add("separated", "right");
      requisites_title.innerText = "Реквизиты:";

      // Инициализация поля ввода
      const requisites_input = document.createElement("input");
      requisites_input.classList.add("medium");
      requisites_input.setAttribute("type", "text");
      requisites_input.setAttribute("minlength", "0");
      requisites_input.setAttribute("maxlength", "250");
      requisites_input.setAttribute(
        "onkeydown",
        "if (event.keyCode === 13) this.parentElement.parentElement.nextElementSibling.nextElementSibling.nextElementSibling.nextElementSibling.children[0].children[0].click()",
      );
      requisites_input.setAttribute("title", "Банковские реквизиты сотрудника");
      requisites_input.setAttribute(
        "placeholder",
        "0000 0000 0000 0000 Банк Иванов И.",
      );

      // Инициализация строки
      const row_payment = document.createElement("div");
      row_payment.classList.add("row", "merged");

      // Инициализация оболочки для строки
      const payment = document.createElement("label");
      payment.setAttribute("id", "payment");

      // Инициализация заголовка для поля ввода
      const payment_title = document.createElement("b");
      payment_title.classList.add("separated", "right");
      payment_title.innerText = "Форма оплаты:";

      // Инициализация поля ввода
      const payment_input = document.createElement("input");
      payment_input.classList.add("medium");
      payment_input.setAttribute("type", "text");
      payment_input.setAttribute("minlength", "0");
      payment_input.setAttribute("maxlength", "80");
      payment_input.setAttribute(
        "onkeydown",
        "if (event.keyCode === 13) this.parentElement.parentElement.nextElementSibling.nextElementSibling.nextElementSibling.nextElementSibling.children[0].children[0].click()",
      );
      payment_input.setAttribute("title", "Форма оплаты сотруднику");
      payment_input.setAttribute("placeholder", "Перевод на карту");

      // Инициализация строки
      const row_tax = document.createElement("div");
      row_tax.classList.add("row", "merged");

      // Инициализация оболочки для строки
      const tax = document.createElement("label");
      tax.setAttribute("id", "tax");

      // Инициализация заголовка для поля ввода
      const tax_title = document.createElement("b");
      tax_title.classList.add("separated", "right");
      tax_title.innerText = "ИНН:";

      // Инициализация поля ввода
      const tax_input = document.createElement("input");
      tax_input.classList.add("medium");
      tax_input.setAttribute("type", "text");
      tax_input.setAttribute("minlength", "0");
      tax_input.setAttribute("maxlength", "12");
      tax_input.setAttribute(
        "onkeydown",
        "if (event.keyCode === 13) this.parentElement.parentElement.nextElementSibling.nextElementSibling.nextElementSibling.nextElementSibling.children[0].children[0].click()",
      );
      tax_input.setAttribute(
        "title",
        "Индивидуальный налоговый номер сотрудника",
      );
      tax_input.setAttribute("placeholder", "000000000000");

      // Инициализация строки
      const row_city = document.createElement("div");
      row_city.classList.add("row", "divided", "merged");

      // Инициализация оболочки для выпадающего списка
      const city = document.createElement("label");
      city.setAttribute("id", "city");

      // Инициализация выпадающего списка
      const city_select = document.createElement("select");
      city_select.setAttribute("title", "Город проживания сотрудника");

      // Инициализация варианта для выбора
      const city_option_1 = document.createElement("option");
      city_option_1.classList.add("value", "Красноярск");
      city_option_1.innerText = "Красноярск";

      // Инициализация варианта для выбора
      const city_option_2 = document.createElement("option");
      city_option_2.classList.add("value", "Железногорск");
      city_option_2.innerText = "Железногорск";
      /*
      // Инициализация варианта для выбора
      const city_option_3 = document.createElement("option");
      city_option_3.classList.add("value", "");
			city_option_3.innerText = "";

      // Инициализация варианта для выбора
      const city_option_4 = document.createElement("option");
      city_option_4.classList.add("value", "");
			city_option_4.innerText = "";

      // Инициализация варианта для выбора
      const city_option_5 = document.createElement("option");
      city_option_5.classList.add("value", "");
			city_option_5.innerText = "";

      // Инициализация варианта для выбора
      const city_option_ = document.createElement("option");
      city_option_.classList.add("value", "");
			city_option_.innerText = ""; */

      // Инициализация строки
      const row_district = document.createElement("div");
      row_district.classList.add("row", "merged");

      // Инициализация оболочки для строки
      const district = document.createElement("label");
      district.setAttribute("id", "district");

      // Инициализация заголовка для поля ввода
      const district_title = document.createElement("b");
      district_title.classList.add("separated", "right");
      district_title.innerText = "Регион:";

      // Инициализация поля ввода
      const district_input = document.createElement("input");
      district_input.classList.add("medium");
      district_input.setAttribute("type", "text");
      district_input.setAttribute("minlength", "0");
      district_input.setAttribute("maxlength", "80");
      district_input.setAttribute(
        "onkeydown",
        "if (event.keyCode === 13) this.parentElement.parentElement.nextElementSibling.nextElementSibling.nextElementSibling.nextElementSibling.children[0].children[0].click()",
      );
      district_input.setAttribute("title", "Регион проживания сотрудника");
      district_input.setAttribute("placeholder", "Северный");

      // Инициализация строки
      const row_address = document.createElement("div");
      row_address.classList.add("row", "merged");

      // Инициализация оболочки для строки
      const address = document.createElement("label");
      address.setAttribute("id", "address");

      // Инициализация заголовка для поля ввода
      const address_title = document.createElement("b");
      address_title.classList.add("separated", "right");
      address_title.innerText = "Адрес:";

      // Инициализация поля ввода
      const address_input = document.createElement("input");
      address_input.classList.add("medium");
      address_input.setAttribute("type", "text");
      address_input.setAttribute("minlength", "0");
      address_input.setAttribute("maxlength", "150");
      address_input.setAttribute(
        "onkeydown",
        "if (event.keyCode === 13) this.parentElement.parentElement.nextElementSibling.nextElementSibling.nextElementSibling.nextElementSibling.children[0].children[0].click()",
      );
      address_input.setAttribute("title", "Адрес проживания сотрудника");
      address_input.setAttribute("placeholder", "ул. Пушкина 7");

      // Инициализация строки
      const row_hiring = document.createElement("div");
      row_hiring.classList.add("row");

      // Инициализация оболочки для строки
      const hiring = document.createElement("label");
      hiring.setAttribute("id", "hiring");

      // Инициализация заголовка для поля ввода
      const hiring_title = document.createElement("b");
      hiring_title.classList.add("separated", "right");
      hiring_title.innerText = "Дата найма:";

      // Инициализация поля ввода
      const hiring_input = document.createElement("input");
      hiring_input.setAttribute("type", "date");
      hiring_input.classList.add("medium");
      hiring_input.setAttribute(
        "onkeydown",
        "if (event.keyCode === 13) this.nextElementSibling.click()",
      );
      hiring_input.setAttribute("title", "Дата найма");
      hiring_input.valueAsDate = new Date();

      // Инициализация строки
      const row_commentary = document.createElement("div");
      row_commentary.classList.add("row");

      // Инициализация оболочки для строки
      const commentary = document.createElement("label");
      commentary.setAttribute("id", "commentary");

      // Инициализация поля ввода
      const commentary_textarea = document.createElement("textarea");
      commentary_textarea.setAttribute("maxlength", "800");
      commentary_textarea.setAttribute(
        "title",
        "Любая дополнительная информация о сотруднике которую увидят только операторы и администраторы",
      );
      commentary_textarea.setAttribute(
        "placeholder",
        "Дополнительная информация о сотруднике",
      );

      // Инициализация строки
      const row_buttons = document.createElement("div");
      row_buttons.classList.add("row", "divided", "buttons");

      // Инициализация оболочки для кнопок
      const buttons = document.createElement("label");

      // Инициализация кнопки подтверждения даты
      const create = document.createElement("button");
      create.classList.add("grass", "stretched");
      create.innerText = "Создать";
      create.setAttribute(
        "onclick",
        `tasks._create(document.getElementById('name_1'), document.getElementById('name_2'), document.getElementById('name_3'), document.getElementById('birth'), document.getElementById('number'), document.getElementById('passport'), document.getElementById('issued'), document.getElementById('department'), document.getElementById('department_2'), document.getElementById('hiring'), document.getElementById('city'), document.getElementById('district'), document.getElementById('address'), document.getElementById('requisites'), document.getElementById('payment'), document.getElementById('tax'), document.getElementById('commentary'), this)`,
      );

      // Инициализация окна с ошибками
      this.body.errors = document.createElement("section");
      this.body.errors.classList.add(
        "errors",
        "window",
        "list",
        "small",
        "hidden",
      );

      // Инициализация элемента-тела (оболочки) окна с ошибками
      const errors = document.createElement("section");
      errors.classList.add("body");

      // Инициализация элемента-списка ошибок
      const dl = document.createElement("dl");

      // Инициализация устаревшего всплывающего окна
      const old = document.getElementById("popup");

      // Удаление устаревшего всплывающего окна
      if (old instanceof HTMLElement) old.remove();

      // Запись в документ
      popup.appendChild(title);

			name.appendChild(name_title);
			name.appendChild(name_input);
      row_name.appendChild(name);
      popup.appendChild(row_name);

			name_2.appendChild(name_2_title);
			name_2.appendChild(name_2_input);
      row_name_2.appendChild(name_2);
      popup.appendChild(row_name_2);

			name_3.appendChild(name_3_title);
			name_3.appendChild(name_3_input);
      row_name_3.appendChild(name_3);
      popup.appendChild(row_name_3);

			birth.appendChild(birth_title);
			birth.appendChild(birth_input);
      row_birth.appendChild(birth);
      popup.appendChild(row_birth);

			number.appendChild(number_title);
			number.appendChild(number_input);
      row_number.appendChild(number);
      popup.appendChild(row_number);

			passport.appendChild(passport_title);
			passport.appendChild(passport_input);
      row_passport.appendChild(passport);
      popup.appendChild(row_passport);

			issued.appendChild(issued_title);
			issued.appendChild(issued_input);
      row_issued.appendChild(issued);
      popup.appendChild(row_issued);

			department.appendChild(department_title);
			department.appendChild(department_input);
      row_department.appendChild(department);
      popup.appendChild(row_department);

			department_2.appendChild(department_2_title);
			department_2.appendChild(department_2_input);
      row_department_2.appendChild(department_2);
      popup.appendChild(row_department_2);

			requisites.appendChild(requisites_title);
			requisites.appendChild(requisites_input);
      row_requisites.appendChild(requisites);
      popup.appendChild(row_requisites);

			payment.appendChild(payment_title);
			payment.appendChild(payment_input);
      row_payment.appendChild(payment);
      popup.appendChild(row_payment);

			tax.appendChild(tax_title);
			tax.appendChild(tax_input);
      row_tax.appendChild(tax);
      popup.appendChild(row_tax);

			city_select.appendChild(city_option_1);
			city_select.appendChild(city_option_2);
			city.appendChild(city_select);
      row_city.appendChild(city);
      popup.appendChild(row_city);

			district.appendChild(district_title);
			district.appendChild(district_input);
      row_district.appendChild(district);
      popup.appendChild(row_district);

			address.appendChild(address_title);
			address.appendChild(address_input);
      row_address.appendChild(address);
      popup.appendChild(row_address);

			hiring.appendChild(hiring_title);
			hiring.appendChild(hiring_input);
      row_hiring.appendChild(hiring);
      popup.appendChild(row_hiring);
/*
			commentary.appendChild(commentary_textarea);
      row_commentary.appendChild(commentary);
      popup.appendChild(row_commentary); */

      buttons.appendChild(create);
      row_buttons.appendChild(buttons);
      popup.appendChild(row_buttons);

      this.body.wrap.appendChild(popup);
      document.body.appendChild(this.body.wrap);

      errors.appendChild(dl);
      this.body.errors.appendChild(errors);
      this.body.wrap.appendChild(this.body.errors);

      // Фокусировка
      name_input.focus();

      // Инициалиация масок
      IMask(number_input, { mask: "+{7} (000) 000-00-00" });
      IMask(passport_input, { mask: "0000 000000" });
      IMask(department_input, { mask: "000-000" });
      IMask(requisites_input, {
        mask:
          /^\d\d\d\d\s\d\d\d\d\s\d\d\d\d\s\d\d\d\d\s[А-Я][а-я]{0,20}\s[А-Я][а-я]{0,30}\s[А-Я]\.$/i,
      });
      IMask(tax_input, { mask: "000000000000" });

      // Инициализация переменных для окна с ошибками (12 - это значение gap из div#popup)
      function top(errors) {
        errors.style.setProperty(
          "transition",
          "0s",
        );
        errors.style.setProperty(
          "--top",
          popup.offsetTop + popup.offsetHeight + 12 + "px",
        );
        setTimeout(
          () => errors.style.removeProperty("transition"),
          100,
        );
      }
      top(this.body.errors);
      const resize = new ResizeObserver(() => top(this.body.errors));
      resize.observe(this.body.wrap);

      /* // Инициализация функции блокировки кнопки
      const block = () => {
        if (
          cashier_input.value > 0 ||
          displayer_input.value > 0 ||
          loader_input.value > 0 ||
          gastronome_input.value > 0
        ) create.removeAttribute("disabled");
        else create.setAttribute("disabled", "true");
      };

      // Добавление функции блокировки кнопки по событиям
      cashier_input.addEventListener("keyup", block);
      displayer_input.addEventListener("keyup", block);
      loader_input.addEventListener("keyup", block);
      gastronome_input.addEventListener("keyup", block);
      cashier_input.addEventListener("change", block);
      displayer_input.addEventListener("change", block);
      loader_input.addEventListener("change", block);
      gastronome_input.addEventListener("change", block); */

      // Первичная активация функции блокировки кнопки
      block();

      // Инициализация функции закрытия всплывающего окна
      const click = () => {
        // Удаление всплывающего окна
        this.body.wrap.remove();
      };

      // Инициализация функции добавления функции закрытия всплывающего окна
      const enable = () => this.body.wrap.addEventListener("click", click);

      // Инициализация функции удаления функции закрытия всплывающего окна
      const disable = () => this.body.wrap.removeEventListener("click", click);

      // Первичная активация функции удаления всплывающего окна
      enable();

      // Добавление функций удаления всплывающего окна по событиям
      popup.addEventListener("mouseenter", disable);
      popup.addEventListener("mouseleave", enable);
    }, 300);

    /**
     * Инициализация страниц
     *
     * @param {number} amount Количество страниц для генерации
     * @param {number} iteration Номер итерации (системное)
     *
     * @return {void}
     */
    static async init(amount = 3, iteration = 0) {
      if (
        typeof amount === "number" && typeof iteration === "number" &&
        this.initialized === false
      ) {
        // Получены количество страниц и номер итерации

        // Инициализация страницы (написал workers потому, что оптимизатор выдаёт ошибку, что this.read не найдено, хотя всё работает)
        await workers.read(++iteration);

        // Проверка условий и отложенный запуск следующей итерации
        if (iteration < amount) setTimeout(this.init, 1500, amount, iteration);
        else this.initialized = true;
      }
    }

    /**
     * Реинициализация страниц
     *
     * @param {function} postprocessing Функция которая будет исполнена после реинициализации
     * @param {number} amount Количество страниц для генерации при инициализации
     *
     * @return {void}
     */
    static reinit = damper((postprocessing, amount = 3) => {
      if (typeof amount === "number") {
        // Получены количество страниц

        // Деинициализация cookie с номером страницы
        Cookies.remove("workers_page", { path: "/" });

        // Инициализация оболочки
        const tasks = document.getElementById("workers");

        if (tasks instanceof HTMLElement) {
          // Найдена оболочка

          // Инициализация буфера элементов
          const buffer = document.createElement("div");

          // Перенос элементов в буфер
          buffer.replaceChildren(...tasks.children);

          // Удаление комментариев с индексами страниц
          tasks.innerHTML = tasks.innerHTML.replace(
            /<!--(?!>)[\S\s]*?-->/g,
            "",
          );

          // Перенос элементов из буфера
          tasks.replaceChildren(...buffer.children);

          // Деинициализация страниц
          for (
            const element of tasks.querySelectorAll(
              ':scope > :not(form, div[id="title"])',
            )
          ) element.remove();

          // Сброс статуса инициализированности
          this.initialized = false;

          // Инициализация страниц
          this.init(amount);

          // Постобработка
          if (typeof postprocessing === "function") postprocessing();
        }
      }
    }, 2000);

    /**
     * Прочитать
     *
     * Читает список сотрудников и модифицирует документ (записывает в общий список)
     *
     * @param {number} page Страница
     *
     * @return {object} {(bool) exist, (array) errors}
     */
    static read = damper(async (page) => {
      if (typeof core === "function" && typeof page !== "number") {
        // Не получена страница

        // Инициализация страницы (если не получена, то брать из cookie и прибавлять 1)
        page = Cookies.get(`workers_page`) ?? 0;

        // Запись идентификатора следующей страницы
        ++page;
      }

      // Запрос к серверу
      return await fetch("/workers/read", {
        method: "POST",
        headers: {
          "Content-Type": "application/x-www-form-urlencoded",
        },
        body: `page=${page}`,
      })
        .then((response) => response.text())
        .then((data) => {
          // Инициализация буфера для записи в документ
          const element = document.createElement("div");

          // Запись пустого элемента из буфера в документ
          document.getElementById("workers").appendChild(element);

          // Запись в документ HTML-данных через буфер
          element.outerHTML = data;

          // Возврат (успех)
          return data;
        });
    }, 50);

    /**
     * Сгенерировать список
     *
     * @return {array|null} Массив HTML-элементов <option>
     */
    static async list() {
      // Запрос к серверу
      return await fetch(`/workers/list`, {
        method: "POST",
        headers: {
          "Content-Type": "application/x-www-form-urlencoded",
        },
      })
        .then((response) => response.text())
        .then((data) => {
          if (data.length > 0) {
            // Получены данные

            return data;
          }

          return null;
        });
    }
  };
}

// Вызов события: "инициализировано"
document.dispatchEvent(
  new CustomEvent("workers.initialized", {
    detail: { workers: window.workers },
  }),
);
